@@ -34,7 +34,6 @@
 #include "ScriptDebugServer.h"
 
 #include "ContentSearchUtils.h"
-#include "EventLoop.h"
 #include "Frame.h"
 #include "JSJavaScriptCallFrame.h"
 #include "JavaScriptCallFrame.h"
@@ -56,10 +55,13 @@ ScriptDebugServer::ScriptDebugServer()
     , m_pauseOnExceptionsState(DontPauseOnExceptions)
     , m_pauseOnNextStatement(false)
     , m_paused(false)
+    , m_runningNestedMessageLoop(false)
     , m_doneProcessingDebuggerEvents(true)
     , m_breakpointsActivated(true)
     , m_pauseOnCallFrame(0)
     , m_recompileTimer(this, &ScriptDebugServer::recompileAllJSFunctions)
+    , m_lastExecutedLine(-1)
+    , m_lastExecutedSourceId(-1)
 {
 }
 
@@ -75,11 +77,11 @@ String ScriptDebugServer::setBreakpoint(const String& sourceID, const ScriptBrea
     SourceIdToBreakpointsMap::iterator it = m_sourceIdToBreakpoints.find(sourceIDValue);
     if (it == m_sourceIdToBreakpoints.end())
         it = m_sourceIdToBreakpoints.set(sourceIDValue, LineToBreakpointMap()).iterator;
-    LineToBreakpointMap::iterator breaksIt = it->second.find(scriptBreakpoint.lineNumber + 1);
-    if (breaksIt == it->second.end())
-        breaksIt = it->second.set(scriptBreakpoint.lineNumber + 1, BreakpointsInLine()).iterator;
+    LineToBreakpointMap::iterator breaksIt = it->value.find(scriptBreakpoint.lineNumber + 1);
+    if (breaksIt == it->value.end())
+        breaksIt = it->value.set(scriptBreakpoint.lineNumber + 1, BreakpointsInLine()).iterator;
 
-    BreakpointsInLine& breaksVector = breaksIt->second;
+    BreakpointsInLine& breaksVector = breaksIt->value;
     unsigned breaksCount = breaksVector.size();
     for (unsigned i = 0; i < breaksCount; i++) {
         if (breaksVector.at(i).columnNumber == scriptBreakpoint.columnNumber)
@@ -112,11 +114,11 @@ void ScriptDebugServer::removeBreakpoint(const String& breakpointId)
     SourceIdToBreakpointsMap::iterator it = m_sourceIdToBreakpoints.find(sourceIDValue);
     if (it == m_sourceIdToBreakpoints.end())
         return;
-    LineToBreakpointMap::iterator breaksIt = it->second.find(lineNumber + 1);
-    if (breaksIt == it->second.end())
+    LineToBreakpointMap::iterator breaksIt = it->value.find(lineNumber + 1);
+    if (breaksIt == it->value.end())
         return;
 
-    BreakpointsInLine& breaksVector = breaksIt->second;
+    BreakpointsInLine& breaksVector = breaksIt->value;
     unsigned breaksCount = breaksVector.size();
     for (unsigned i = 0; i < breaksCount; i++) {
         if (breaksVector.at(i).columnNumber == static_cast<int>(columnNumber)) {
@@ -126,44 +128,6 @@ void ScriptDebugServer::removeBreakpoint(const String& breakpointId)
     }
 }
 
-void ScriptDebugServer::updateCurrentStatementPosition(intptr_t sourceID, int line)
-{
-    if (line < 0)
-        return;
-
-    SourceProvider* source = reinterpret_cast<SourceProvider*>(sourceID);
-
-    if (m_currentSourceID != sourceID) {
-        String sourceCode = ustringToString(JSC::UString(const_cast<StringImpl*>(source->data())));
-        m_currentSourceCode.clear();
-        sourceCode.split("\n", true, m_currentSourceCode);
-        m_currentSourceID = sourceID;
-        m_currentStatementPosition.lineNumber = 0;
-        m_currentStatementPosition.columnNumber = 0;
-    }
-
-    if (line != m_currentStatementPosition.lineNumber) {
-        m_currentStatementPosition.lineNumber = line;
-        m_currentStatementPosition.columnNumber = 0;
-        return;
-    }
-
-    int startLine = source->startPosition().m_line.zeroBasedInt();
-    if ((m_currentStatementPosition.lineNumber - startLine - 1) >= static_cast<int>(m_currentSourceCode.size()))
-        return;
-    const String& codeInLine = m_currentSourceCode[m_currentStatementPosition.lineNumber - startLine - 1];
-    if (codeInLine.isEmpty())
-        return;
-    int nextColumn = codeInLine.find(";", m_currentStatementPosition.columnNumber);
-    if (nextColumn != -1) {
-        UChar c = codeInLine[nextColumn + 1];
-        if (c == ' ' || c == '\t')
-            nextColumn += 1;
-        m_currentStatementPosition.columnNumber = nextColumn + 1;
-    } else
-        m_currentStatementPosition.columnNumber = 0;
-}
-
 bool ScriptDebugServer::hasBreakpoint(intptr_t sourceID, const TextPosition& position) const
 {
     if (!m_breakpointsActivated)
@@ -178,17 +142,20 @@ bool ScriptDebugServer::hasBreakpoint(intptr_t sourceID, const TextPosition& pos
     if (lineNumber < 0 || columnNumber < 0)
         return false;
 
-    LineToBreakpointMap::const_iterator breaksIt = it->second.find(lineNumber + 1);
-    if (breaksIt == it->second.end())
+    LineToBreakpointMap::const_iterator breaksIt = it->value.find(lineNumber + 1);
+    if (breaksIt == it->value.end())
         return false;
 
     bool hit = false;
-    const BreakpointsInLine& breaksVector = breaksIt->second;
+    const BreakpointsInLine& breaksVector = breaksIt->value;
     unsigned breaksCount = breaksVector.size();
     unsigned i;
     for (i = 0; i < breaksCount; i++) {
         int breakLine = breaksVector.at(i).lineNumber;
-        if (lineNumber == breakLine) {
+        int breakColumn = breaksVector.at(i).columnNumber;
+        // Since frontend truncates the indent, the first statement in a line must match the breakpoint (line,0).
+        if ((lineNumber != m_lastExecutedLine && lineNumber == breakLine && !breakColumn)
+            || (lineNumber == breakLine && columnNumber == breakColumn)) {
             hit = true;
             break;
         }
@@ -201,12 +168,12 @@ bool ScriptDebugServer::hasBreakpoint(intptr_t sourceID, const TextPosition& pos
         return true;
 
     JSValue exception;
-    JSValue result = m_currentCallFrame->evaluate(stringToUString(breaksVector.at(i).condition), exception);
+    JSValue result = m_currentCallFrame->evaluate(breaksVector.at(i).condition, exception);
     if (exception) {
         // An erroneous condition counts as "false".
         return false;
     }
-    return result.toBoolean();
+    return result.toBoolean(m_currentCallFrame->exec());
 }
 
 void ScriptDebugServer::clearBreakpoints()
@@ -295,7 +262,7 @@ void ScriptDebugServer::updateCallStack(ScriptValue*)
 void ScriptDebugServer::dispatchDidPause(ScriptDebugListener* listener)
 {
     ASSERT(m_paused);
-    JSGlobalObject* globalObject = m_currentCallFrame->scopeChain()->globalObject.get();
+    JSGlobalObject* globalObject = m_currentCallFrame->scopeChain()->globalObject();
     ScriptState* state = globalObject->globalExec();
     JSValue jsCallFrame;
     {
@@ -316,18 +283,21 @@ void ScriptDebugServer::dispatchDidContinue(ScriptDebugListener* listener)
 
 void ScriptDebugServer::dispatchDidParseSource(const ListenerSet& listeners, SourceProvider* sourceProvider, bool isContentScript)
 {
-    String sourceID = ustringToString(JSC::UString::number(sourceProvider->asID()));
+    String sourceID = String::number(sourceProvider->asID());
 
     ScriptDebugListener::Script script;
-    script.url = ustringToString(sourceProvider->url());
-    script.source = ustringToString(JSC::UString(const_cast<StringImpl*>(sourceProvider->data())));
+    script.url = sourceProvider->url();
+    script.source = sourceProvider->source();
     script.startLine = sourceProvider->startPosition().m_line.zeroBasedInt();
     script.startColumn = sourceProvider->startPosition().m_column.zeroBasedInt();
     script.isContentScript = isContentScript;
 
 #if ENABLE(INSPECTOR)
-    if (script.url.isEmpty())
-        script.url = ContentSearchUtils::findSourceURL(script.source);
+    if (!script.startLine && !script.startColumn) {
+        String sourceURL = ContentSearchUtils::findSourceURL(script.source);
+        if (!sourceURL.isEmpty())
+            script.url = sourceURL;
+    }
 #endif
 
     int sourceLength = script.source.length();
@@ -354,8 +324,8 @@ void ScriptDebugServer::dispatchDidParseSource(const ListenerSet& listeners, Sou
 
 void ScriptDebugServer::dispatchFailedToParseSource(const ListenerSet& listeners, SourceProvider* sourceProvider, int errorLine, const String& errorMessage)
 {
-    String url = ustringToString(sourceProvider->url());
-    String data = ustringToString(JSC::UString(const_cast<StringImpl*>(sourceProvider->data())));
+    String url = sourceProvider->url();
+    const String& data = sourceProvider->source();
     int firstLine = sourceProvider->startPosition().m_line.oneBasedInt();
 
     Vector<ScriptDebugListener*> copy;
@@ -364,7 +334,7 @@ void ScriptDebugServer::dispatchFailedToParseSource(const ListenerSet& listeners
         copy[i]->failedToParseSource(url, data, firstLine, errorLine, errorMessage);
 }
 
-static bool isContentScript(ExecState* exec)
+bool ScriptDebugServer::isContentScript(ExecState* exec)
 {
     return currentWorld(exec) != mainThreadNormalWorld();
 }
@@ -382,7 +352,7 @@ void ScriptDebugServer::detach(JSGlobalObject* globalObject)
     Debugger::detach(globalObject);
 }
 
-void ScriptDebugServer::sourceParsed(ExecState* exec, SourceProvider* sourceProvider, int errorLine, const UString& errorMessage)
+void ScriptDebugServer::sourceParsed(ExecState* exec, SourceProvider* sourceProvider, int errorLine, const String& errorMessage)
 {
     if (m_callingListeners)
         return;
@@ -396,7 +366,7 @@ void ScriptDebugServer::sourceParsed(ExecState* exec, SourceProvider* sourceProv
 
     bool isError = errorLine != -1;
     if (isError)
-        dispatchFailedToParseSource(*listeners, sourceProvider, errorLine, ustringToString(errorMessage));
+        dispatchFailedToParseSource(*listeners, sourceProvider, errorLine, errorMessage);
     else
         dispatchDidParseSource(*listeners, sourceProvider, isContentScript(exec));
 
@@ -426,11 +396,14 @@ void ScriptDebugServer::dispatchFunctionToListeners(JavaScriptExecutionCallback
     m_callingListeners = false;
 }
 
-void ScriptDebugServer::createCallFrameAndPauseIfNeeded(const DebuggerCallFrame& debuggerCallFrame, intptr_t sourceID, int lineNumber, int columnNumber)
+void ScriptDebugServer::createCallFrame(const DebuggerCallFrame& debuggerCallFrame, intptr_t sourceID, int lineNumber, int columnNumber)
 {
     TextPosition textPosition(OrdinalNumber::fromOneBasedInt(lineNumber), OrdinalNumber::fromZeroBasedInt(columnNumber));
     m_currentCallFrame = JavaScriptCallFrame::create(debuggerCallFrame, m_currentCallFrame, sourceID, textPosition);
-    pauseIfNeeded(debuggerCallFrame.dynamicGlobalObject());
+    if (m_lastExecutedSourceId != sourceID) {
+        m_lastExecutedLine = -1;
+        m_lastExecutedSourceId = sourceID;
+    }
 }
 
 void ScriptDebugServer::updateCallFrameAndPauseIfNeeded(const DebuggerCallFrame& debuggerCallFrame, intptr_t sourceID, int lineNumber, int columnNumber)
@@ -455,6 +428,7 @@ void ScriptDebugServer::pauseIfNeeded(JSGlobalObject* dynamicGlobalObject)
     bool pauseNow = m_pauseOnNextStatement;
     pauseNow |= (m_pauseOnCallFrame == m_currentCallFrame);
     pauseNow |= hasBreakpoint(m_currentCallFrame->sourceID(), m_currentCallFrame->position());
+    m_lastExecutedLine = m_currentCallFrame->position().m_line.zeroBasedInt();
     if (!pauseNow)
         return;
 
@@ -467,10 +441,10 @@ void ScriptDebugServer::pauseIfNeeded(JSGlobalObject* dynamicGlobalObject)
 
     TimerBase::fireTimersInNestedEventLoop();
 
-    EventLoop loop;
+    m_runningNestedMessageLoop = true;
     m_doneProcessingDebuggerEvents = false;
-    while (!m_doneProcessingDebuggerEvents && !loop.ended())
-        loop.cycle();
+    runEventLoopWhilePaused();
+    m_runningNestedMessageLoop = false;
 
     didContinue(dynamicGlobalObject);
     dispatchFunctionToListeners(&ScriptDebugServer::dispatchDidContinue, dynamicGlobalObject);
@@ -480,8 +454,10 @@ void ScriptDebugServer::pauseIfNeeded(JSGlobalObject* dynamicGlobalObject)
 
 void ScriptDebugServer::callEvent(const DebuggerCallFrame& debuggerCallFrame, intptr_t sourceID, int lineNumber, int columnNumber)
 {
-    if (!m_paused)
-        createCallFrameAndPauseIfNeeded(debuggerCallFrame, sourceID, lineNumber, columnNumber);
+    if (!m_paused) {
+        createCallFrame(debuggerCallFrame, sourceID, lineNumber, columnNumber);
+        pauseIfNeeded(debuggerCallFrame.dynamicGlobalObject());
+    }
 }
 
 void ScriptDebugServer::atStatement(const DebuggerCallFrame& debuggerCallFrame, intptr_t sourceID, int lineNumber, int columnNumber)
@@ -520,8 +496,10 @@ void ScriptDebugServer::exception(const DebuggerCallFrame& debuggerCallFrame, in
 
 void ScriptDebugServer::willExecuteProgram(const DebuggerCallFrame& debuggerCallFrame, intptr_t sourceID, int lineNumber, int columnNumber)
 {
-    if (!m_paused)
-        createCallFrameAndPauseIfNeeded(debuggerCallFrame, sourceID, lineNumber, columnNumber);
+    if (!m_paused) {
+        createCallFrame(debuggerCallFrame, sourceID, lineNumber, columnNumber);
+        pauseIfNeeded(debuggerCallFrame.dynamicGlobalObject());
+    }
 }
 
 void ScriptDebugServer::didExecuteProgram(const DebuggerCallFrame& debuggerCallFrame, intptr_t sourceID, int lineNumber, int columnNumber)
@@ -532,8 +510,13 @@ void ScriptDebugServer::didExecuteProgram(const DebuggerCallFrame& debuggerCallF
     updateCallFrameAndPauseIfNeeded(debuggerCallFrame, sourceID, lineNumber, columnNumber);
 
     // Treat stepping over the end of a program like stepping out.
-    if (m_currentCallFrame == m_pauseOnCallFrame)
+    if (!m_currentCallFrame)
+        return;
+    if (m_currentCallFrame == m_pauseOnCallFrame) {
         m_pauseOnCallFrame = m_currentCallFrame->caller();
+        if (!m_currentCallFrame)
+            return;
+    }
     m_currentCallFrame = m_currentCallFrame->caller();
 }
 
