@@ -33,6 +33,7 @@
 #include <wtf/HashMap.h>
 #include <wtf/StdLibExtras.h>
 #include <wtf/Vector.h>
+#include <wtf/text/StringBuilder.h>
 
 namespace WebCore {
 
@@ -40,6 +41,7 @@ using namespace HTMLNames;
 
 void CSSSelector::createRareData()
 {
+    ASSERT(m_match != Tag);
     if (m_hasRareData)
         return;
     // Move the value to the rare data stucture.
@@ -51,11 +53,27 @@ unsigned CSSSelector::specificity() const
 {
     // make sure the result doesn't overflow
     static const unsigned maxValueMask = 0xffffff;
+    static const unsigned idMask = 0xff0000;
+    static const unsigned classMask = 0xff00;
+    static const unsigned elementMask = 0xff;
+
+    if (isForPage())
+        return specificityForPage() & maxValueMask;
+
     unsigned total = 0;
+    unsigned temp = 0;
+
     for (const CSSSelector* selector = this; selector; selector = selector->tagHistory()) {
-        if (selector->m_isForPage)
-            return (total + selector->specificityForPage()) & maxValueMask;
-        total = (total + selector->specificityForOneSelector()) & maxValueMask;
+        temp = total + selector->specificityForOneSelector();
+        // Clamp each component to its max in the case of overflow.
+        if ((temp & idMask) < (total & idMask))
+            total |= idMask;
+        else if ((temp & classMask) < (total & classMask))
+            total |= classMask;
+        else if ((temp & elementMask) < (total & elementMask))
+            total |= elementMask;
+        else
+            total = temp;
     }
     return total;
 }
@@ -64,11 +82,9 @@ inline unsigned CSSSelector::specificityForOneSelector() const
 {
     // FIXME: Pseudo-elements and pseudo-classes do not have the same specificity. This function
     // isn't quite correct.
-    unsigned s = (m_tag.localName() == starAtom ? 0 : 1);
     switch (m_match) {
     case Id:
-        s += 0x10000;
-        break;
+        return 0x10000;
     case Exact:
     case Class:
     case Set:
@@ -82,32 +98,45 @@ inline unsigned CSSSelector::specificityForOneSelector() const
         // FIXME: PsuedoAny should base the specificity on the sub-selectors.
         // See http://lists.w3.org/Archives/Public/www-style/2010Sep/0530.html
         if (pseudoType() == PseudoNot && selectorList())
-            s += selectorList()->first()->specificityForOneSelector();
-        else
-            s += 0x100;
-    case None:
-        break;
+            return selectorList()->first()->specificityForOneSelector();
+        return 0x100;
+    case Tag:
+        return (tagQName().localName() != starAtom) ? 1 : 0;
+    case Unknown:
+        return 0;
     }
-    return s;
+    ASSERT_NOT_REACHED();
+    return 0;
 }
 
 unsigned CSSSelector::specificityForPage() const
 {
     // See http://dev.w3.org/csswg/css3-page/#cascading-and-page-context
-    unsigned s = (m_tag.localName() == starAtom ? 0 : 4);
+    unsigned s = 0;
 
-    switch (pseudoType()) {
-    case PseudoFirstPage:
-        s += 2;
-        break;
-    case PseudoLeftPage:
-    case PseudoRightPage:
-        s += 1;
-        break;
-    case PseudoNotParsed:
-        break;
-    default:
-        ASSERT_NOT_REACHED();
+    for (const CSSSelector* component = this; component; component = component->tagHistory()) {
+        switch (component->m_match) {
+        case Tag:
+            s += tagQName().localName() == starAtom ? 0 : 4;
+            break;
+        case PseudoClass:
+            switch (component->pseudoType()) {
+            case PseudoFirstPage:
+                s += 2;
+                break;
+            case PseudoLeftPage:
+            case PseudoRightPage:
+                s += 1;
+                break;
+            case PseudoNotParsed:
+                break;
+            default:
+                ASSERT_NOT_REACHED();
+            }
+            break;
+        default:
+            break;
+        }
     }
     return s;
 }
@@ -182,7 +211,6 @@ PseudoId CSSSelector::pseudoId(PseudoType type)
     case PseudoValid:
     case PseudoInvalid:
     case PseudoIndeterminate:
-    case PseudoScope:
     case PseudoTarget:
     case PseudoLang:
     case PseudoNot:
@@ -205,6 +233,16 @@ PseudoId CSSSelector::pseudoId(PseudoType type)
     case PseudoRightPage:
     case PseudoInRange:
     case PseudoOutOfRange:
+    case PseudoUserAgentCustomElement:
+    case PseudoWebKitCustomElement:
+#if ENABLE(VIDEO_TRACK)
+    case PseudoCue:
+    case PseudoFutureCue:
+    case PseudoPastCue:
+#endif
+#if ENABLE(IFRAME_SEAMLESS)
+    case PseudoSeamlessDocument:
+#endif
         return NOPSEUDO;
     case PseudoNotParsed:
         ASSERT_NOT_REACHED();
@@ -264,7 +302,6 @@ static HashMap<AtomicStringImpl*, CSSSelector::PseudoType>* nameToPseudoTypeMap(
     DEFINE_STATIC_LOCAL(AtomicString, scrollbarTrack, ("-webkit-scrollbar-track", AtomicString::ConstructFromLiteral));
     DEFINE_STATIC_LOCAL(AtomicString, scrollbarTrackPiece, ("-webkit-scrollbar-track-piece", AtomicString::ConstructFromLiteral));
     DEFINE_STATIC_LOCAL(AtomicString, selection, ("selection", AtomicString::ConstructFromLiteral));
-    DEFINE_STATIC_LOCAL(AtomicString, scope, ("scope", AtomicString::ConstructFromLiteral));
     DEFINE_STATIC_LOCAL(AtomicString, target, ("target", AtomicString::ConstructFromLiteral));
     DEFINE_STATIC_LOCAL(AtomicString, visited, ("visited", AtomicString::ConstructFromLiteral));
     DEFINE_STATIC_LOCAL(AtomicString, windowInactive, ("window-inactive", AtomicString::ConstructFromLiteral));
@@ -287,6 +324,14 @@ static HashMap<AtomicStringImpl*, CSSSelector::PseudoType>* nameToPseudoTypeMap(
     DEFINE_STATIC_LOCAL(AtomicString, fullScreenDocument, ("-webkit-full-screen-document", AtomicString::ConstructFromLiteral));
     DEFINE_STATIC_LOCAL(AtomicString, fullScreenAncestor, ("-webkit-full-screen-ancestor", AtomicString::ConstructFromLiteral));
     DEFINE_STATIC_LOCAL(AtomicString, animatingFullScreenTransition, ("-webkit-animating-full-screen-transition", AtomicString::ConstructFromLiteral));
+#endif
+#if ENABLE(VIDEO_TRACK)
+    DEFINE_STATIC_LOCAL(AtomicString, cue, ("cue(", AtomicString::ConstructFromLiteral));
+    DEFINE_STATIC_LOCAL(AtomicString, futureCue, ("future", AtomicString::ConstructFromLiteral));
+    DEFINE_STATIC_LOCAL(AtomicString, pastCue, ("past", AtomicString::ConstructFromLiteral));
+#endif
+#if ENABLE(IFRAME_SEAMLESS)
+    DEFINE_STATIC_LOCAL(AtomicString, seamlessDocument, ("-webkit-seamless-document", AtomicString::ConstructFromLiteral));
 #endif
     DEFINE_STATIC_LOCAL(AtomicString, inRange, ("in-range", AtomicString::ConstructFromLiteral));
     DEFINE_STATIC_LOCAL(AtomicString, outOfRange, ("out-of-range", AtomicString::ConstructFromLiteral));
@@ -352,7 +397,6 @@ static HashMap<AtomicStringImpl*, CSSSelector::PseudoType>* nameToPseudoTypeMap(
         nameToPseudoType->set(scrollbarTrackPiece.impl(), CSSSelector::PseudoScrollbarTrackPiece);
         nameToPseudoType->set(cornerPresent.impl(), CSSSelector::PseudoCornerPresent);
         nameToPseudoType->set(selection.impl(), CSSSelector::PseudoSelection);
-        nameToPseudoType->set(scope.impl(), CSSSelector::PseudoScope);
         nameToPseudoType->set(target.impl(), CSSSelector::PseudoTarget);
         nameToPseudoType->set(visited.impl(), CSSSelector::PseudoVisited);
         nameToPseudoType->set(firstPage.impl(), CSSSelector::PseudoFirstPage);
@@ -363,6 +407,14 @@ static HashMap<AtomicStringImpl*, CSSSelector::PseudoType>* nameToPseudoTypeMap(
         nameToPseudoType->set(fullScreenDocument.impl(), CSSSelector::PseudoFullScreenDocument);
         nameToPseudoType->set(fullScreenAncestor.impl(), CSSSelector::PseudoFullScreenAncestor);
         nameToPseudoType->set(animatingFullScreenTransition.impl(), CSSSelector::PseudoAnimatingFullScreenTransition);
+#endif
+#if ENABLE(VIDEO_TRACK)
+        nameToPseudoType->set(cue.impl(), CSSSelector::PseudoCue);
+        nameToPseudoType->set(futureCue.impl(), CSSSelector::PseudoFutureCue);
+        nameToPseudoType->set(pastCue.impl(), CSSSelector::PseudoPastCue);
+#endif
+#if ENABLE(IFRAME_SEAMLESS)
+        nameToPseudoType->set(seamlessDocument.impl(), CSSSelector::PseudoSeamlessDocument);
 #endif
         nameToPseudoType->set(inRange.impl(), CSSSelector::PseudoInRange);
         nameToPseudoType->set(outOfRange.impl(), CSSSelector::PseudoOutOfRange);
@@ -376,12 +428,22 @@ CSSSelector::PseudoType CSSSelector::parsePseudoType(const AtomicString& name)
         return PseudoUnknown;
     HashMap<AtomicStringImpl*, CSSSelector::PseudoType>* nameToPseudoType = nameToPseudoTypeMap();
     HashMap<AtomicStringImpl*, CSSSelector::PseudoType>::iterator slot = nameToPseudoType->find(name.impl());
-    return slot == nameToPseudoType->end() ? PseudoUnknown : slot->second;
+
+    if (slot != nameToPseudoType->end())
+        return slot->value;
+
+    if (name.startsWith("-webkit-"))
+        return PseudoWebKitCustomElement;
+    if (name.startsWith("x-") || name.startsWith("cue"))
+        return PseudoUserAgentCustomElement;
+
+    return PseudoUnknown;
 }
 
-bool CSSSelector::isUnknownPseudoType(const AtomicString& name)
+bool CSSSelector::isCustomPseudoType(const AtomicString& name)
 {
-    return parsePseudoType(name) == PseudoUnknown;
+    CSSSelector::PseudoType type = parsePseudoType(name);
+    return type == PseudoUserAgentCustomElement || type == PseudoWebKitCustomElement;
 }
 
 void CSSSelector::extractPseudoType() const
@@ -398,6 +460,9 @@ void CSSSelector::extractPseudoType() const
     switch (m_pseudoType) {
     case PseudoAfter:
     case PseudoBefore:
+#if ENABLE(VIDEO_TRACK)
+    case PseudoCue:
+#endif
     case PseudoFirstLetter:
     case PseudoFirstLine:
         compat = true;
@@ -409,6 +474,8 @@ void CSSSelector::extractPseudoType() const
     case PseudoScrollbarTrack:
     case PseudoScrollbarTrackPiece:
     case PseudoSelection:
+    case PseudoUserAgentCustomElement:
+    case PseudoWebKitCustomElement:
         element = true;
         break;
     case PseudoUnknown:
@@ -444,7 +511,6 @@ void CSSSelector::extractPseudoType() const
     case PseudoValid:
     case PseudoInvalid:
     case PseudoIndeterminate:
-    case PseudoScope:
     case PseudoTarget:
     case PseudoLang:
     case PseudoNot:
@@ -468,9 +534,16 @@ void CSSSelector::extractPseudoType() const
     case PseudoFullScreenDocument:
     case PseudoFullScreenAncestor:
     case PseudoAnimatingFullScreenTransition:
+#endif
+#if ENABLE(IFRAME_SEAMLESS)
+    case PseudoSeamlessDocument:
 #endif
     case PseudoInRange:
     case PseudoOutOfRange:
+#if ENABLE(VIDEO_TRACK)
+    case PseudoFutureCue:
+    case PseudoPastCue:
+#endif
         break;
     case PseudoFirstPage:
     case PseudoLeftPage:
@@ -491,18 +564,24 @@ void CSSSelector::extractPseudoType() const
         m_pseudoType = PseudoUnknown;
 }
 
-bool CSSSelector::operator==(const CSSSelector& other)
+bool CSSSelector::operator==(const CSSSelector& other) const
 {
     const CSSSelector* sel1 = this;
     const CSSSelector* sel2 = &other;
 
     while (sel1 && sel2) {
-        if (sel1->m_tag != sel2->m_tag || sel1->attribute() != sel2->attribute() ||
-             sel1->relation() != sel2->relation() || sel1->m_match != sel2->m_match ||
-             sel1->value() != sel2->value() ||
-             sel1->pseudoType() != sel2->pseudoType() ||
-             sel1->argument() != sel2->argument())
+        if (sel1->attribute() != sel2->attribute()
+            || sel1->relation() != sel2->relation()
+            || sel1->m_match != sel2->m_match
+            || sel1->value() != sel2->value()
+            || sel1->pseudoType() != sel2->pseudoType()
+            || sel1->argument() != sel2->argument()) {
             return false;
+        }
+        if (sel1->m_match == Tag) {
+            if (sel1->tagQName() != sel2->tagQName())
+                return false;
+        }
         sel1 = sel1->tagHistory();
         sel2 = sel2->tagHistory();
     }
@@ -515,99 +594,97 @@ bool CSSSelector::operator==(const CSSSelector& other)
 
 String CSSSelector::selectorText() const
 {
-    String str = "";
+    StringBuilder str;
 
-    const AtomicString& prefix = m_tag.prefix();
-    const AtomicString& localName = m_tag.localName();
-    if (m_match == CSSSelector::None || !prefix.isNull() || localName != starAtom) {
-        if (prefix.isNull())
-            str = localName;
+    if (m_match == CSSSelector::Tag && !m_tagIsForNamespaceRule) {
+        if (tagQName().prefix().isNull())
+            str.append(tagQName().localName());
         else {
-            str = prefix.string();
-            str.append("|");
-            str.append(localName);
+            str.append(tagQName().prefix().string());
+            str.append('|');
+            str.append(tagQName().localName());
         }
     }
 
     const CSSSelector* cs = this;
     while (true) {
         if (cs->m_match == CSSSelector::Id) {
-            str += "#";
+            str.append('#');
             serializeIdentifier(cs->value(), str);
         } else if (cs->m_match == CSSSelector::Class) {
-            str += ".";
+            str.append('.');
             serializeIdentifier(cs->value(), str);
         } else if (cs->m_match == CSSSelector::PseudoClass || cs->m_match == CSSSelector::PagePseudoClass) {
-            str += ":";
-            str += cs->value();
+            str.append(':');
+            str.append(cs->value());
 
             switch (cs->pseudoType()) {
             case PseudoNot:
-                if (CSSSelectorList* selectorList = cs->selectorList())
-                    str += selectorList->first()->selectorText();
-                str += ")";
+                if (const CSSSelectorList* selectorList = cs->selectorList())
+                    str.append(selectorList->first()->selectorText());
+                str.append(')');
                 break;
             case PseudoLang:
             case PseudoNthChild:
             case PseudoNthLastChild:
             case PseudoNthOfType:
             case PseudoNthLastOfType:
-                str += cs->argument();
-                str += ")";
+                str.append(cs->argument());
+                str.append(')');
                 break;
             case PseudoAny: {
-                CSSSelector* firstSubSelector = cs->selectorList()->first();
-                for (CSSSelector* subSelector = firstSubSelector; subSelector; subSelector = CSSSelectorList::next(subSelector)) {
+                const CSSSelector* firstSubSelector = cs->selectorList()->first();
+                for (const CSSSelector* subSelector = firstSubSelector; subSelector; subSelector = CSSSelectorList::next(subSelector)) {
                     if (subSelector != firstSubSelector)
-                        str += ",";
-                    str += subSelector->selectorText();
+                        str.append(',');
+                    str.append(subSelector->selectorText());
                 }
-                str += ")";
+                str.append(')');
                 break;
             }
             default:
                 break;
             }
         } else if (cs->m_match == CSSSelector::PseudoElement) {
-            str += "::";
-            str += cs->value();
+            str.appendLiteral("::");
+            str.append(cs->value());
         } else if (cs->isAttributeSelector()) {
-            str += "[";
+            str.append('[');
             const AtomicString& prefix = cs->attribute().prefix();
             if (!prefix.isNull()) {
                 str.append(prefix);
                 str.append("|");
             }
-            str += cs->attribute().localName();
+            str.append(cs->attribute().localName());
             switch (cs->m_match) {
                 case CSSSelector::Exact:
-                    str += "=";
+                    str.append('=');
                     break;
                 case CSSSelector::Set:
                     // set has no operator or value, just the attrName
-                    str += "]";
+                    str.append(']');
                     break;
                 case CSSSelector::List:
-                    str += "~=";
+                    str.appendLiteral("~=");
                     break;
                 case CSSSelector::Hyphen:
-                    str += "|=";
+                    str.appendLiteral("|=");
                     break;
                 case CSSSelector::Begin:
-                    str += "^=";
+                    str.appendLiteral("^=");
                     break;
                 case CSSSelector::End:
-                    str += "$=";
+                    str.appendLiteral("$=");
                     break;
                 case CSSSelector::Contain:
-                    str += "*=";
+                    str.appendLiteral("*=");
                     break;
                 default:
                     break;
             }
             if (cs->m_match != CSSSelector::Set) {
                 serializeString(cs->value(), str);
-                str += "]";
+                str.append(']');
             }
         }
         if (cs->relation() != CSSSelector::SubSelector || !cs->tagHistory())
@@ -615,22 +692,25 @@ String CSSSelector::selectorText() const
         cs = cs->tagHistory();
     }
 
-    if (CSSSelector* tagHistory = cs->tagHistory()) {
+    if (const CSSSelector* tagHistory = cs->tagHistory()) {
         String tagHistoryText = tagHistory->selectorText();
-        if (cs->relation() == CSSSelector::DirectAdjacent)
-            str = tagHistoryText + " + " + str;
-        else if (cs->relation() == CSSSelector::IndirectAdjacent)
-            str = tagHistoryText + " ~ " + str;
-        else if (cs->relation() == CSSSelector::Child)
-            str = tagHistoryText + " > " + str;
-        else if (cs->relation() == CSSSelector::ShadowDescendant)
-            str = tagHistoryText + str;
-        else
-            // Descendant
-            str = tagHistoryText + " " + str;
+        switch (cs->relation()) {
+        case CSSSelector::Descendant:
+            return tagHistoryText + " " + str.toString();
+        case CSSSelector::Child:
+            return tagHistoryText + " > " + str.toString();
+        case CSSSelector::DirectAdjacent:
+            return tagHistoryText + " + " + str.toString();
+        case CSSSelector::IndirectAdjacent:
+            return tagHistoryText + " ~ " + str.toString();
+        case CSSSelector::SubSelector:
+            ASSERT_NOT_REACHED();
+        case CSSSelector::ShadowDescendant:
+            return tagHistoryText + str.toString();
+        }
     }
 
-    return str;
+    return str.toString();
 }
 
 void CSSSelector::setAttribute(const QualifiedName& value)
@@ -651,7 +731,7 @@ void CSSSelector::setSelectorList(PassOwnPtr<CSSSelectorList> selectorList)
     m_data.m_rareData->m_selectorList = selectorList;
 }
 
-bool CSSSelector::parseNth()
+bool CSSSelector::parseNth() const
 {
     if (!m_hasRareData)
         return false;
@@ -661,37 +741,12 @@ bool CSSSelector::parseNth()
     return m_parsedNth;
 }
 
-bool CSSSelector::matchNth(int count)
+bool CSSSelector::matchNth(int count) const
 {
     ASSERT(m_hasRareData);
     return m_data.m_rareData->matchNth(count);
 }
 
-bool CSSSelector::isSimple() const
-{
-    if (selectorList() || tagHistory() || matchesPseudoElement())
-        return false;
-
-    int numConditions = 0;
-
-    // hasTag() cannot be be used here because namespace may not be nullAtom.
-    // Example:
-    //     @namespace "http://www.w3.org/2000/svg";
-    //     svg:not(:root) { ...
-    if (m_tag != starAtom)
-        numConditions++;
-
-    if (m_match == Id || m_match == Class || m_match == PseudoClass)
-        numConditions++;
-
-    if (m_hasRareData && m_data.m_rareData->m_attribute != anyQName())
-        numConditions++;
-
-    // numConditions is 0 for a universal selector.
-    // numConditions is 1 for other simple selectors.
-    return numConditions <= 1;
-}
-
 CSSSelector::RareData::RareData(PassRefPtr<AtomicStringImpl> value)
     : m_value(value.leakRef())
     , m_a(0)
