@@ -30,13 +30,13 @@
 #include "Element.h"
 #include "HTMLNames.h"
 #include "InlineTextBox.h"
+#include "NodeTraversal.h"
 #include "Position.h"
 #include "RenderBlock.h"
 #include "RenderObject.h"
 #include "RenderedPosition.h"
 #include "Text.h"
 #include "TextBoundaries.h"
-#include "TextBreakIterator.h"
 #include "TextIterator.h"
 #include "VisiblePosition.h"
 #include "VisibleSelection.h"
@@ -215,7 +215,7 @@ static const InlineTextBox* logicallyPreviousBox(const VisiblePosition& visibleP
         return previousBox;
 
     while (1) {
-        Node* startNode = startBox->renderer() ? startBox->renderer()->node() : 0;
+        Node* startNode = startBox->renderer() ? startBox->renderer()->nonPseudoNode() : 0;
         if (!startNode)
             break;
 
@@ -256,7 +256,7 @@ static const InlineTextBox* logicallyNextBox(const VisiblePosition& visiblePosit
         return nextBox;
 
     while (1) {
-        Node* startNode = startBox->renderer() ? startBox->renderer()->node() : 0;
+        Node* startNode = startBox->renderer() ? startBox->renderer()->nonPseudoNode() : 0;
         if (!startNode)
             break;
 
@@ -744,7 +744,7 @@ static VisiblePosition startPositionForLine(const VisiblePosition& c, LineEndpoi
             if (!startRenderer)
                 return VisiblePosition();
 
-            startNode = startRenderer->node();
+            startNode = startRenderer->nonPseudoNode();
             if (startNode)
                 break;
 
@@ -816,7 +816,7 @@ static VisiblePosition endPositionForLine(const VisiblePosition& c, LineEndpoint
             if (!endRenderer)
                 return VisiblePosition();
 
-            endNode = endRenderer->node();
+            endNode = endRenderer->nonPseudoNode();
             if (endNode)
                 break;
             
@@ -1109,22 +1109,26 @@ VisiblePosition startOfParagraph(const VisiblePosition& c, EditingBoundaryCrossi
 
     Node* n = startNode;
     while (n) {
+#if ENABLE(USERSELECT_ALL)
+        if (boundaryCrossingRule == CannotCrossEditingBoundary && !Position::nodeIsUserSelectAll(n) && n->rendererIsEditable() != startNode->rendererIsEditable())
+#else
         if (boundaryCrossingRule == CannotCrossEditingBoundary && n->rendererIsEditable() != startNode->rendererIsEditable())
+#endif
             break;
         if (boundaryCrossingRule == CanSkipOverEditingBoundary) {
             while (n && n->rendererIsEditable() != startNode->rendererIsEditable())
-                n = n->traversePreviousNodePostOrder(startBlock);
+                n = NodeTraversal::previousPostOrder(n, startBlock);
             if (!n || !n->isDescendantOf(highestRoot))
                 break;
         }
         RenderObject *r = n->renderer();
         if (!r) {
-            n = n->traversePreviousNodePostOrder(startBlock);
+            n = NodeTraversal::previousPostOrder(n, startBlock);
             continue;
         }
         RenderStyle *style = r->style();
         if (style->visibility() != VISIBLE) {
-            n = n->traversePreviousNodePostOrder(startBlock);
+            n = NodeTraversal::previousPostOrder(n, startBlock);
             continue;
         }
         
@@ -1132,7 +1136,7 @@ VisiblePosition startOfParagraph(const VisiblePosition& c, EditingBoundaryCrossi
             break;
 
         if (r->isText() && toRenderText(r)->renderedTextLength()) {
-            ASSERT(n->isTextNode());
+            ASSERT_WITH_SECURITY_IMPLICATION(n->isTextNode());
             type = Position::PositionIsOffsetInAnchor;
             if (style->preserveNewline()) {
                 const UChar* chars = toRenderText(r)->characters();
@@ -1147,13 +1151,13 @@ VisiblePosition startOfParagraph(const VisiblePosition& c, EditingBoundaryCrossi
             }
             node = n;
             offset = 0;
-            n = n->traversePreviousNodePostOrder(startBlock);
+            n = NodeTraversal::previousPostOrder(n, startBlock);
         } else if (editingIgnoresContent(n) || isTableElement(n)) {
             node = n;
             type = Position::PositionIsBeforeAnchor;
-            n = n->previousSibling() ? n->previousSibling() : n->traversePreviousNodePostOrder(startBlock);
+            n = n->previousSibling() ? n->previousSibling() : NodeTraversal::previousPostOrder(n, startBlock);
         } else
-            n = n->traversePreviousNodePostOrder(startBlock);
+            n = NodeTraversal::previousPostOrder(n, startBlock);
     }
 
     if (type == Position::PositionIsOffsetInAnchor) {
@@ -1185,23 +1189,27 @@ VisiblePosition endOfParagraph(const VisiblePosition &c, EditingBoundaryCrossing
 
     Node* n = startNode;
     while (n) {
+#if ENABLE(USERSELECT_ALL)
+        if (boundaryCrossingRule == CannotCrossEditingBoundary && !Position::nodeIsUserSelectAll(n) && n->rendererIsEditable() != startNode->rendererIsEditable())
+#else
         if (boundaryCrossingRule == CannotCrossEditingBoundary && n->rendererIsEditable() != startNode->rendererIsEditable())
+#endif
             break;
         if (boundaryCrossingRule == CanSkipOverEditingBoundary) {
             while (n && n->rendererIsEditable() != startNode->rendererIsEditable())
-                n = n->traverseNextNode(stayInsideBlock);
+                n = NodeTraversal::next(n, stayInsideBlock);
             if (!n || !n->isDescendantOf(highestRoot))
                 break;
         }
 
         RenderObject *r = n->renderer();
         if (!r) {
-            n = n->traverseNextNode(stayInsideBlock);
+            n = NodeTraversal::next(n, stayInsideBlock);
             continue;
         }
         RenderStyle *style = r->style();
         if (style->visibility() != VISIBLE) {
-            n = n->traverseNextNode(stayInsideBlock);
+            n = NodeTraversal::next(n, stayInsideBlock);
             continue;
         }
         
@@ -1210,7 +1218,7 @@ VisiblePosition endOfParagraph(const VisiblePosition &c, EditingBoundaryCrossing
 
         // FIXME: We avoid returning a position where the renderer can't accept the caret.
         if (r->isText() && toRenderText(r)->renderedTextLength()) {
-            ASSERT(n->isTextNode());
+            ASSERT_WITH_SECURITY_IMPLICATION(n->isTextNode());
             int length = toRenderText(r)->textLength();
             type = Position::PositionIsOffsetInAnchor;
             if (style->preserveNewline()) {
@@ -1223,13 +1231,13 @@ VisiblePosition endOfParagraph(const VisiblePosition &c, EditingBoundaryCrossing
             }
             node = n;
             offset = r->caretMaxOffset();
-            n = n->traverseNextNode(stayInsideBlock);
+            n = NodeTraversal::next(n, stayInsideBlock);
         } else if (editingIgnoresContent(n) || isTableElement(n)) {
             node = n;
             type = Position::PositionIsAfterAnchor;
-            n = n->traverseNextSibling(stayInsideBlock);
+            n = NodeTraversal::nextSkippingChildren(n, stayInsideBlock);
         } else
-            n = n->traverseNextNode(stayInsideBlock);
+            n = NodeTraversal::next(n, stayInsideBlock);
     }
 
     if (type == Position::PositionIsOffsetInAnchor)
