@@ -63,7 +63,6 @@ function defineCommonExtensionSymbols(apiPrivate)
         ConsoleMessageAdded: "console-message-added",
         ElementsPanelObjectSelected: "panel-objectSelected-elements",
         NetworkRequestFinished: "network-request-finished",
-        Reset: "reset",
         OpenResource: "open-resource",
         PanelSearch: "panel-search-",
         Reload: "Reload",
@@ -97,6 +96,7 @@ function defineCommonExtensionSymbols(apiPrivate)
         ShowPanel: "showPanel",
         StopAuditCategoryRun: "stopAuditCategoryRun",
         Unsubscribe: "unsubscribe",
+        UpdateAuditProgress: "updateAuditProgress",
         UpdateButton: "updateButton",
         InspectedURLChanged: "inspectedURLChanged"
     };
@@ -182,8 +182,6 @@ function InspectorExtensionAPI()
     defineDeprecatedProperty(this, "webInspector", "resources", "network");
     this.timeline = new Timeline();
     this.console = new ConsoleAPI();
-
-    this.onReset = new EventSink(events.Reset);
 }
 
 /**
@@ -377,10 +375,10 @@ PanelWithSidebarImpl.prototype = {
             callback(new ExtensionSidebarPane(id));
         }
         extensionServer.sendRequest(request, callback && callbackWrapper);
-    }
-}
+    },
 
-PanelWithSidebarImpl.prototype.__proto__ = ExtensionViewImpl.prototype;
+    __proto__: ExtensionViewImpl.prototype
+}
 
 /**
  * @constructor
@@ -429,10 +427,10 @@ ExtensionPanelImpl.prototype = {
             id: this._id
         };
         extensionServer.sendRequest(request);
-    }
-};
+    },
 
-ExtensionPanelImpl.prototype.__proto__ = ExtensionViewImpl.prototype;
+    __proto__: ExtensionViewImpl.prototype
+}
 
 /**
  * @constructor
@@ -509,6 +507,8 @@ Audits.prototype = {
     addCategory: function(displayName, resultCount)
     {
         var id = "extension-audit-category-" + extensionServer.nextObjectId();
+        if (typeof resultCount !== "undefined")
+            console.warn("Passing resultCount to audits.addCategory() is deprecated. Use AuditResult.updateProgress() instead.");
         extensionServer.sendRequest({ command: commands.AddAuditCategory, id: id, displayName: displayName, resultCount: resultCount });
         return new AuditCategory(id);
     }
@@ -570,6 +570,11 @@ AuditResultImpl.prototype = {
         return new AuditResultNode(Array.prototype.slice.call(arguments));
     },
 
+    updateProgress: function(worked, totalWork)
+    {
+        extensionServer.sendRequest({ command: commands.UpdateAuditProgress, resultId: this._id, progress: worked / totalWork });
+    },
+
     done: function()
     {
         extensionServer.sendRequest({ command: commands.StopAuditCategoryRun, resultId: this._id });
@@ -742,6 +747,9 @@ function ExtensionServerClient()
 }
 
 ExtensionServerClient.prototype = {
+    /**
+     * @param {function()=} callback
+     */
     sendRequest: function(message, callback)
     {
         if (typeof callback === "function")
