@@ -28,12 +28,15 @@
 #include "KURL.h"
 
 #include "DecodeEscapeSequences.h"
+#include "MIMETypeRegistry.h"
+#include "PlatformMemoryInstrumentation.h"
 #include "TextEncoding.h"
 #include <stdio.h>
 #include <wtf/HashMap.h>
 #if !USE(WTFURL)
 #include <wtf/HexNumber.h>
 #endif
+#include <wtf/MemoryInstrumentationString.h>
 #include <wtf/StdLibExtras.h>
 #include <wtf/text/CString.h>
 #include <wtf/text/StringBuilder.h>
@@ -41,8 +44,6 @@
 
 #if USE(ICU_UNICODE)
 #include <unicode/uidna.h>
-#elif USE(QT4_UNICODE)
-#include <QUrl>
 #elif USE(GLIB_UNICODE)
 #include <glib.h>
 #include <wtf/gobject/GOwnPtr.h>
@@ -293,7 +294,7 @@ static void appendASCII(const String& base, const char* rel, size_t len, CharBuf
     buffer[buffer.size() - 1] = '\0';
 }
 
-// FIXME: Move to PlatformString.h eventually.
+// FIXME: Move to WTFString.h eventually.
 // Returns the index of the first index in string |s| of any of the characters
 // in |toFind|. |toFind| should be a null-terminated string, all characters up
 // to the null will be searched. Returns int if not found.
@@ -1484,9 +1485,6 @@ static void appendEncodedHostname(UCharBuffer& buffer, const UChar* str, unsigne
         hostnameBufferLength, UIDNA_ALLOW_UNASSIGNED, 0, &error);
     if (error == U_ZERO_ERROR)
         buffer.append(hostnameBuffer, numCharactersConverted);
-#elif USE(QT4_UNICODE)
-    QByteArray result = QUrl::toAce(String(str, strLen));
-    buffer.append(result.constData(), result.length());
 #elif USE(GLIB_UNICODE)
     GOwnPtr<gchar> utf8Hostname;
     GOwnPtr<GError> utf8Err;
@@ -1916,4 +1914,25 @@ String mimeTypeFromDataURL(const String& url)
     return "";
 }
 
+String mimeTypeFromURL(const KURL& url)
+{
+    String decodedPath = decodeURLEscapeSequences(url.path());
+    String extension = decodedPath.substring(decodedPath.reverseFind('.') + 1);
+
+    // We don't use MIMETypeRegistry::getMIMETypeForPath() because it returns "application/octet-stream" upon failure
+    return MIMETypeRegistry::getMIMETypeForExtension(extension);
+}
+
+void KURL::reportMemoryUsage(MemoryObjectInfo* memoryObjectInfo) const
+{
+    MemoryClassInfo info(memoryObjectInfo, this);
+#if USE(GOOGLEURL)
+    info.addMember(m_url, "url");
+#elif USE(WTFURL)
+    info.addMember(m_urlImpl, "urlImpl");
+#else // !USE(GOOGLEURL)
+    info.addMember(m_string, "string");
+#endif
+}
+
 }
