@@ -56,7 +56,7 @@ static void gtkStyleChangedCallback(GObject*, GParamSpec*)
 {
     StyleContextMap::const_iterator end = styleContextMap().end();
     for (StyleContextMap::const_iterator iter = styleContextMap().begin(); iter != end; ++iter)
-        gtk_style_context_invalidate(iter->second.get());
+        gtk_style_context_invalidate(iter->value.get());
 
     Page::scheduleForcedStyleRecalcForAllPages();
 }
@@ -79,7 +79,7 @@ static GtkStyleContext* getStyleContext(GType widgetType)
 {
     StyleContextMap::AddResult result = styleContextMap().add(widgetType, 0);
     if (!result.isNewEntry)
-        return result.iterator->second.get();
+        return result.iterator->value.get();
 
     GtkWidgetPath* path = gtk_widget_path_new();
     gtk_widget_path_append_type(path, widgetType);
@@ -111,7 +111,7 @@ static GtkStyleContext* getStyleContext(GType widgetType)
     gtk_style_context_set_path(context.get(), path);
     gtk_widget_path_free(path);
 
-    result.iterator->second = context;
+    result.iterator->value = context;
     return context.get();
 }
 
@@ -767,7 +767,8 @@ bool RenderThemeGtk::paintProgressBar(RenderObject* renderObject, const PaintInf
 
 static gint spinButtonArrowSize(GtkStyleContext* context)
 {
-    const PangoFontDescription* fontDescription = gtk_style_context_get_font(context, static_cast<GtkStateFlags>(0));
+    const PangoFontDescription* fontDescription;
+    gtk_style_context_get(context, static_cast<GtkStateFlags>(0), "font", &fontDescription, NULL);
     gint fontSize = pango_font_description_get_size(fontDescription);
     gint arrowSize = max(PANGO_PIXELS(fontSize), minSpinButtonArrowSize);
 
