@@ -29,6 +29,7 @@
 
 #include "BlobRegistry.h"
 #include "Logging.h"
+#include "NetworkingContext.h"
 #include "ResourceHandleClient.h"
 #include "Timer.h"
 #include <algorithm>
@@ -71,7 +72,7 @@ PassRefPtr<ResourceHandle> ResourceHandle::create(NetworkingContext* context, co
     BuiltinResourceHandleConstructorMap::iterator protocolMapItem = builtinResourceHandleConstructorMap().find(request.url().protocol());
 
     if (protocolMapItem != builtinResourceHandleConstructorMap().end())
-        return protocolMapItem->second(request, client);
+        return protocolMapItem->value(request, client);
 
     RefPtr<ResourceHandle> newHandle(adoptRef(new ResourceHandle(request, client, defersLoading, shouldContentSniff)));
 
@@ -184,6 +185,11 @@ void ResourceHandle::setDefersLoading(bool defers)
     platformSetDefersLoading(defers);
 }
 
+void ResourceHandle::didChangePriority(ResourceLoadPriority)
+{
+    // Optionally implemented by platform.
+}
+
 void ResourceHandle::cacheMetadata(const ResourceResponse&, const Vector<char>&)
 {
     // Optionally implemented by platform.
