@@ -311,11 +311,7 @@ void RenderSVGResourceFilter::postApplyResource(RenderObject* object, GraphicsCo
             filterData->state = FilterData::Applying;
             lastEffect->apply();
             lastEffect->correctFilterResultIfNeeded();
-#if !USE(CG)
-            ImageBuffer* resultImage = lastEffect->asImageBuffer();
-            if (resultImage)
-                resultImage->transformColorSpace(lastEffect->colorSpace(), ColorSpaceDeviceRGB);
-#endif
+            lastEffect->transformResultColorSpace(ColorSpaceDeviceRGB);
         }
         filterData->state = FilterData::Built;
 
@@ -348,7 +344,7 @@ void RenderSVGResourceFilter::primitiveAttributeChanged(RenderObject* object, co
     SVGFilterPrimitiveStandardAttributes* primitve = static_cast<SVGFilterPrimitiveStandardAttributes*>(object->node());
 
     for (; it != end; ++it) {
-        FilterData* filterData = it->second;
+        FilterData* filterData = it->value;
         if (filterData->state != FilterData::Built)
             continue;
 
@@ -363,8 +359,9 @@ void RenderSVGResourceFilter::primitiveAttributeChanged(RenderObject* object, co
         builder->clearResultsRecursive(effect);
 
         // Repaint the image on the screen.
-        markClientForInvalidation(it->first, RepaintInvalidation);
+        markClientForInvalidation(it->key, RepaintInvalidation);
     }
+    markAllClientLayersForInvalidation();
 }
 
 }
