@@ -27,6 +27,7 @@
 
 #include "LauncherInspectorWindow.h"
 #include <errno.h>
+#include <gst/gst.h>
 #include <gtk/gtk.h>
 #include <stdlib.h>
 #include <string.h>
@@ -487,8 +488,10 @@ int main(int argc, char* argv[])
     GOptionContext *context = g_option_context_new(0);
     g_option_context_add_main_entries(context, commandLineOptions, 0);
     g_option_context_add_group(context, gtk_get_option_group(TRUE));
+    g_option_context_add_group(context, gst_init_get_option_group());
 
     webkitSettings = webkit_web_settings_new();
+    g_object_set(webkitSettings, "enable-developer-extras", TRUE, NULL);
     if (!addWebSettingsGroupToContext(context, webkitSettings)) {
         g_object_unref(webkitSettings);
         webkitSettings = 0;
@@ -515,6 +518,10 @@ int main(int argc, char* argv[])
     }
 #endif
 
+#ifdef WEBKIT_EXEC_PATH
+    g_setenv("WEBKIT_INSPECTOR_PATH", WEBKIT_EXEC_PATH "resources/inspector", FALSE);
+#endif /* WEBKIT_EXEC_PATH */
+
     WebKitWebView *webView;
     GtkWidget *main_window = createWindow(&webView);
 
