@@ -80,6 +80,7 @@ my $numberOfCPUs;
 my $baseProductDir;
 my @baseProductDirOption;
 my $configuration;
+my $xcodeSDK;
 my $configurationForVisualStudio;
 my $configurationProductDir;
 my $sourceDir;
@@ -222,7 +223,7 @@ sub determineBaseProductDir
 
         undef $baseProductDir unless $baseProductDir =~ /^\//;
     } elsif (isChromium()) {
-        if (isLinux() || isChromiumAndroid() || isChromiumMacMake()) {
+        if (isLinux() || isChromiumAndroid() || isChromiumMacMake() || isChromiumNinja()) {
             $baseProductDir = "$sourceDir/out";
         } elsif (isDarwin()) {
             $baseProductDir = "$sourceDir/Source/WebKit/chromium/xcodebuild";
@@ -301,6 +302,7 @@ sub determineArchitecture
     $architecture = "";
 
     determineBaseProductDir();
+    determineXcodeSDK();
 
     if (isGtk()) {
         determineConfigurationProductDir();
@@ -317,9 +319,15 @@ sub determineArchitecture
         if ($architecture) {
             chomp $architecture;
         } else {
-            my $supports64Bit = `sysctl -n hw.optional.x86_64`;
-            chomp $supports64Bit;
-            $architecture = 'x86_64' if $supports64Bit;
+            if (not defined $xcodeSDK or $xcodeSDK =~ /^(\/$|macosx)/) {
+                my $supports64Bit = `sysctl -n hw.optional.x86_64`;
+                chomp $supports64Bit;
+                $architecture = 'x86_64' if $supports64Bit;
+            } elsif ($xcodeSDK =~ /^iphonesimulator/) {
+                $architecture = 'i386';
+            } elsif ($xcodeSDK =~ /^iphoneos/) {
+                $architecture = 'armv7';
+            }
         }
     } elsif (isEfl()) {
         my $host_processor = "";
@@ -396,6 +404,30 @@ sub argumentsForConfiguration()
     return @args;
 }
 
+sub determineXcodeSDK
+{
+    return if defined $xcodeSDK;
+    for (my $i = 0; $i <= $#ARGV; $i++) {
+        my $opt = $ARGV[$i];
+        if ($opt =~ /^--sdk$/i) {
+            splice(@ARGV, $i, 1);
+            $xcodeSDK = splice(@ARGV, $i, 1);
+        } elsif ($opt =~ /^--device$/i) {
+            splice(@ARGV, $i, 1);
+            $xcodeSDK = 'iphoneos.internal';
+        } elsif ($opt =~ /^--sim(ulator)?/i) {
+            splice(@ARGV, $i, 1);
+            $xcodeSDK = 'iphonesimulator';
+        }
+    }
+}
+
+sub xcodeSDK
+{
+    determineXcodeSDK();
+    return $xcodeSDK;
+}
+
 sub determineConfigurationForVisualStudio
 {
     return if defined $configurationForVisualStudio;
@@ -520,7 +552,12 @@ sub XcodeOptions
     determineBaseProductDir();
     determineConfiguration();
     determineArchitecture();
-    return (@baseProductDirOption, "-configuration", $configuration, "ARCHS=$architecture", argumentsForXcode());
+    determineXcodeSDK();
+
+    my @sdkOption = ($xcodeSDK ? "SDKROOT=$xcodeSDK" : ());
+    my @architectureOption = ($architecture ? "ARCHS=$architecture" : ());
+
+    return (@baseProductDirOption, "-configuration", $configuration, @architectureOption, @sdkOption, argumentsForXcode());
 }
 
 sub XcodeOptionString
@@ -723,7 +760,11 @@ sub builtDylibPathForName
     }
     if (isQt()) {
         my $isSearchingForWebCore = $libraryName =~ "WebCore";
-        $libraryName = "QtWebKit";
+        if (isDarwin()) {
+            $libraryName = "QtWebKitWidgets";
+        } else {
+            $libraryName = "Qt5WebKitWidgets";
+        }
         my $result;
         if (isDarwin() and -d "$configurationProductDir/lib/$libraryName.framework") {
             $result = "$configurationProductDir/lib/$libraryName.framework/$libraryName";
@@ -735,8 +776,8 @@ sub builtDylibPathForName
                 $libraryName .= "d";
             }
 
-            my $mkspec = `$qmakebin -query QMAKE_MKSPECS`;
-            $mkspec =~ s/[\n|\r]$//g;
+            chomp(my $mkspec = `$qmakebin -query QT_HOST_DATA`);
+            $mkspec .= "/mkspecs";
             my $qtMajorVersion = retrieveQMakespecVar("$mkspec/qconfig.pri", "QT_MAJOR_VERSION");
             if (not $qtMajorVersion) {
                 $qtMajorVersion = "";
@@ -774,6 +815,9 @@ sub builtDylibPathForName
         return "NotFound";
     }
     if (isEfl()) {
+        if (isWK2()) {
+            return "$configurationProductDir/lib/libewebkit2.so";
+        }
         return "$configurationProductDir/lib/libewebkit.so";
     }
     if (isWinCE()) {
@@ -834,45 +878,27 @@ sub qtFeatureDefaults
 {
     die "ERROR: qmake missing but required to build WebKit.\n" if not commandExists($qmakebin);
 
-    my $qmakepath = File::Spec->catfile(sourceDir(), "Tools", "qmake");
-    my $qmakecommand;
-    if (isWindows()) {
-        $qmakecommand = "(set QMAKEPATH=$qmakepath) && $qmakebin";
-    } else {
-        $qmakecommand = "QMAKEPATH=$qmakepath $qmakebin";
-    }
+    my $oldQmakeEval = $ENV{QMAKE_CACHE_EVAL};
+    $ENV{QMAKE_CACHE_EVAL} = "CONFIG+=print_defaults";
 
     my $originalCwd = getcwd();
+    my $qmakepath = File::Spec->catfile(sourceDir(), "Tools", "qmake");
+    chdir $qmakepath or die "Failed to cd into " . $qmakepath . "\n";
+
+    my $file = File::Spec->catfile(sourceDir(), "WebKit.pro");
 
-    my $file = File::Spec->catfile($qmakepath, "configure.pro");
     my @buildArgs;
-    my $qconfigs;
+    @buildArgs = (@buildArgs, @{$_[0]}) if (@_);
 
-    if (@_) {
-        @buildArgs = (@buildArgs, @{$_[0]});
-        $qconfigs = $_[1];
-        my $dir = File::Spec->catfile(productDir(), "Tools", "qmake");
-        File::Path::mkpath($dir);
-        chdir $dir or die "Failed to cd into " . $dir . "\n";
-    } else {
-        # Do a quick check of the features without running the config tests
-        push @buildArgs, "CONFIG+=quick_check";
-    }
-
-    my @defaults = `$qmakecommand @buildArgs -nocache $file 2>&1`;
+    my @defaults = `$qmakebin @buildArgs $file 2>&1`;
 
     my %qtFeatureDefaults;
     for (@defaults) {
-        if (/ DEFINES: /) {
+        if (/DEFINES: /) {
             while (/(\S+?)=(\S+?)/gi) {
                 $qtFeatureDefaults{$1}=$2;
             }
-        } elsif (/ CONFIG:(.*)$/) {
-            if (@_) {
-                $$qconfigs = $1;
-            }
         } elsif (/Done computing defaults/) {
-            print "\n";
             last;
         } elsif (@_) {
             print $_;
@@ -880,6 +906,7 @@ sub qtFeatureDefaults
     }
 
     chdir $originalCwd;
+    $ENV{QMAKE_CACHE_EVAL} = $oldQmakeEval;
 
     return %qtFeatureDefaults;
 }
@@ -1022,6 +1049,8 @@ sub blackberryCMakeArguments()
     push @includeSystemDirectories, File::Spec->catdir($stageInc, "grskia");
     push @includeSystemDirectories, File::Spec->catdir($stageInc, "harfbuzz");
     push @includeSystemDirectories, File::Spec->catdir($stageInc, "imf");
+    # We only use jpeg-turbo for device build
+    push @includeSystemDirectories, File::Spec->catdir($stageInc, "jpeg-turbo") if $arch=~/arm/;
     push @includeSystemDirectories, $stageInc;
     push @includeSystemDirectories, File::Spec->catdir($stageInc, "browser", "platform");
     push @includeSystemDirectories, File::Spec->catdir($stageInc, "browser", "qsk");
@@ -1208,7 +1237,11 @@ sub determineIsChromiumNinja()
 {
     return if defined($isChromiumNinja);
 
-    my $config = configuration();
+    # This function can be called from baseProductDir(), which in turn is
+    # called by configuration(). So calling configuration() here leads to
+    # infinite recursion. Gyp writes both Debug and Release at the same time
+    # by default, so just check the timestamp on the Release build.ninja file.
+    my $config = "Release";
 
     my $hasUpToDateNinjabuild = 0;
     if (-e "out/$config/build.ninja") {
@@ -1224,7 +1257,12 @@ sub determineIsChromiumNinja()
           $statMake = stat('Makefile.chromium')->mtime;
         }
 
-        $hasUpToDateNinjabuild = $statNinja > $statXcode && $statNinja > $statMake;
+        my $statVisualStudio = 0;
+        if (-e 'Source/WebKit/chromium/All.sln') {
+          $statVisualStudio = stat('Source/WebKit/chromium/All.sln')->mtime;
+        }
+
+        $hasUpToDateNinjabuild = $statNinja > $statXcode && $statNinja > $statMake && $statNinja > $statVisualStudio;
     }
     $isChromiumNinja = $hasUpToDateNinjabuild;
 }
@@ -1254,7 +1292,7 @@ sub isCygwin()
 
 sub isAnyWindows()
 {
-    return isWindows() || isCygwin() || isMsys();
+    return isWindows() || isCygwin();
 }
 
 sub determineWinVersion()
@@ -1307,11 +1345,6 @@ sub isWindows()
     return ($^O eq "MSWin32") || 0;
 }
 
-sub isMsys()
-{
-    return ($^O eq "msys") || 0;
-}
-
 sub isLinux()
 {
     return ($^O eq "linux") || 0;
@@ -1324,7 +1357,7 @@ sub isFreeBSD()
 
 sub isARM()
 {
-    return $Config{archname} =~ /^arm-/;
+    return $Config{archname} =~ /^arm[v\-]/;
 }
 
 sub isCrossCompilation()
@@ -1463,11 +1496,21 @@ sub debugger
 sub determineDebugger
 {
     return if defined($debugger);
-    if (checkForArgumentAndRemoveFromARGV("--use-lldb")) {
+
+    determineXcodeVersion();
+    if (eval "v$xcodeVersion" ge v4.5) {
         $debugger = "lldb";
     } else {
         $debugger = "gdb";
     }
+
+    if (checkForArgumentAndRemoveFromARGV("--use-lldb")) {
+        $debugger = "lldb";
+    }
+
+    if (checkForArgumentAndRemoveFromARGV("--use-gdb")) {
+        $debugger = "gdb";
+    }
 }
 
 sub appendToEnvironmentVariableList
@@ -1493,9 +1536,6 @@ sub setUpGuardMallocIfNeeded
 
     if ($shouldUseGuardMalloc) {
         appendToEnvironmentVariableList("DYLD_INSERT_LIBRARIES", "/usr/lib/libgmalloc.dylib");
-        if (shouldUseXPCServiceForWebProcess()) {
-            appendToEnvironmentVariableList("__XPC_DYLD_INSERT_LIBRARIES", "/usr/lib/libgmalloc.dylib");
-        }
     }
 }
 
@@ -1529,7 +1569,7 @@ sub launcherName()
     } elsif (isAppleWebKit()) {
         return "Safari";
     } elsif (isEfl()) {
-        return "EWebLauncher";
+        return "EWebLauncher/MiniBrowser";
     } elsif (isWinCE()) {
         return "WinCELauncher";
     }
@@ -1559,8 +1599,21 @@ sub checkRequiredSystemConfig
             print "*************************************************************\n";
         }
     } elsif (isGtk() or isQt() or isWx() or isEfl()) {
-        my @cmds = qw(flex bison gperf);
+        my @cmds = qw(bison gperf);
+        if (isQt() and isWindows()) {
+            push @cmds, "win_flex";
+        } else {
+            push @cmds, "flex";
+        }
         my @missing = ();
+        my $oldPath = $ENV{PATH};
+        if (isQt() and isWindows()) {
+            chomp(my $gnuWin32Dir = `$qmakebin -query QT_HOST_DATA`);
+            $gnuWin32Dir = File::Spec->catfile($gnuWin32Dir, "..", "gnuwin32", "bin");
+            if (-d "$gnuWin32Dir") {
+                $ENV{PATH} = $gnuWin32Dir . ";" . $ENV{PATH};
+            }
+        }
         foreach my $cmd (@cmds) {
             push @missing, $cmd if not commandExists($cmd);
         }
@@ -1569,6 +1622,9 @@ sub checkRequiredSystemConfig
             my $list = join ", ", @missing;
             die "ERROR: $list missing but required to build WebKit.\n";
         }
+        if (isQt() and isWindows()) {
+            $ENV{PATH} = $oldPath;
+        }
     }
     # Win32 and other platforms may want to check for minimum config
 }
@@ -1606,9 +1662,9 @@ sub setupAppleWinEnv()
 
         # FIXME: We should remove this explicit version check for cygwin once we stop supporting Cygwin 1.7.9 or older versions. 
         # https://bugs.webkit.org/show_bug.cgi?id=85791
-        my $currentCygwinVersion = version->parse(`uname -r`);
-        my $firstCygwinVersionWithoutTTYSupport = version->parse("1.7.10");
-        if ($currentCygwinVersion < $firstCygwinVersionWithoutTTYSupport) {
+        my $uname_version = (POSIX::uname())[2];
+        $uname_version =~ s/\(.*\)//;  # Remove the trailing cygwin version, if any.
+        if (version->parse($uname_version) < version->parse("1.7.10")) {
             # Setting the environment variable 'CYGWIN' to 'tty' makes cygwin enable extra support (i.e., termios)
             # for UNIX-like ttys in the Windows console
             $variablesToSet{CYGWIN} = "tty" unless $ENV{CYGWIN};
@@ -1872,9 +1928,13 @@ sub retrieveQMakespecVar
 sub qtMakeCommand($)
 {
     my ($qmakebin) = @_;
-    chomp(my $mkspec = `$qmakebin -query QMAKE_MKSPECS`);
-    $mkspec .= "/default";
-    my $compiler = retrieveQMakespecVar("$mkspec/qmake.conf", "QMAKE_CC");
+    chomp(my $hostDataPath = `$qmakebin -query QT_HOST_DATA`);
+    my $mkspecPath = $hostDataPath . "/mkspecs/default/qmake.conf";
+    if (! -e $mkspecPath) {
+        chomp(my $mkspec= `$qmakebin -query QMAKE_XSPEC`);
+        $mkspecPath = $hostDataPath . "/mkspecs/" . $mkspec . "/qmake.conf";
+    }
+    my $compiler = retrieveQMakespecVar($mkspecPath, "QMAKE_CC");
 
     #print "default spec: " . $mkspec . "\n";
     #print "compiler found: " . $compiler . "\n";
@@ -1896,9 +1956,10 @@ sub autotoolsFlag($$)
 
 sub runAutogenForAutotoolsProjectIfNecessary($@)
 {
-    my ($dir, $prefix, $sourceDir, $project, @buildArgs) = @_;
+    my ($dir, $prefix, $sourceDir, $project, $joinedOverridableFeatures, @buildArgs) = @_;
+
+    my $joinedBuildArgs = join(" ", @buildArgs);
 
-    my $argumentsFile = "previous-autogen-arguments.txt";
     if (-e "GNUmakefile") {
         # Just assume that build-jsc will never be used to reconfigure JSC. Later
         # we can go back and make this more complicated if the demand is there.
@@ -1906,8 +1967,9 @@ sub runAutogenForAutotoolsProjectIfNecessary($@)
             return;
         }
 
-        # We only run autogen.sh again if the arguments passed have changed.
-        if (!mustReRunAutogen($sourceDir, $argumentsFile, @buildArgs)) {
+        # Run autogen.sh again if either the features overrided by build-webkit or build arguments have changed.
+        if (!mustReRunAutogen($sourceDir, "feature-defines-overriding.txt", $joinedOverridableFeatures)
+            && !mustReRunAutogen($sourceDir, "previous-autogen-arguments.txt", $joinedBuildArgs)) {
             return;
         }
     }
@@ -1918,8 +1980,12 @@ sub runAutogenForAutotoolsProjectIfNecessary($@)
     # Only for WebKit, write the autogen.sh arguments to a file so that we can detect
     # when they change and automatically re-run it.
     if ($project eq 'WebKit') {
-        open(AUTOTOOLS_ARGUMENTS, ">$argumentsFile");
-        print AUTOTOOLS_ARGUMENTS join(" ", @buildArgs);
+        open(OVERRIDABLE_FEATURES, ">feature-defines-overriding.txt");
+        print OVERRIDABLE_FEATURES $joinedOverridableFeatures;
+        close(OVERRIDABLE_FEATURES);
+
+        open(AUTOTOOLS_ARGUMENTS, ">previous-autogen-arguments.txt");
+        print AUTOTOOLS_ARGUMENTS $joinedBuildArgs;
         close(AUTOTOOLS_ARGUMENTS);
     }
 
@@ -1950,24 +2016,24 @@ sub getJhbuildPath()
 
 sub mustReRunAutogen($@)
 {
-    my ($sourceDir, $filename, @currentArguments) = @_;
+    my ($sourceDir, $filename, $currentContents) = @_;
 
     if (! -e $filename) {
         return 1;
     }
 
-    open(AUTOTOOLS_ARGUMENTS, $filename);
-    chomp(my $previousArguments = <AUTOTOOLS_ARGUMENTS>);
-    close(AUTOTOOLS_ARGUMENTS);
+    open(CONTENTS_FILE, $filename);
+    chomp(my $previousContents = <CONTENTS_FILE>);
+    close(CONTENTS_FILE);
 
     # We only care about the WebKit2 argument when we are building WebKit itself.
     # build-jsc never passes --enable-webkit2, so if we didn't do this, autogen.sh
     # would run for every single build on the bots, since it runs both build-webkit
     # and build-jsc.
-    my $joinedCurrentArguments = join(" ", @currentArguments);
-    if ($previousArguments ne $joinedCurrentArguments) {
-        print "Previous autogen arguments were: $previousArguments\n\n";
-        print "New autogen arguments are: $joinedCurrentArguments\n";
+    if ($previousContents ne $currentContents) {
+        print "Contents for file $filename have changed.\n";
+        print "Previous contents were: $previousContents\n\n";
+        print "New contents are: $currentContents\n";
         return 1;
     }
 
@@ -1976,12 +2042,11 @@ sub mustReRunAutogen($@)
 
 sub buildAutotoolsProject($@)
 {
-    my ($project, $clean, @buildParams) = @_;
+    my ($project, $clean, $prefix, $makeArgs, $noWebKit1, $noWebKit2, @features) = @_;
 
     my $make = 'make';
     my $dir = productDir();
     my $config = passedConfiguration() || configuration();
-    my $prefix;
 
     # Use rm to clean the build directory since distclean may miss files
     if ($clean && -d $dir) {
@@ -1997,19 +2062,39 @@ sub buildAutotoolsProject($@)
         return 0;
     }
 
-    my @buildArgs = ();
-    my $makeArgs = $ENV{"WebKitMakeArguments"} || "";
-    for my $i (0 .. $#buildParams) {
-        my $opt = $buildParams[$i];
-        if ($opt =~ /^--makeargs=(.*)/i ) {
-            $makeArgs = $makeArgs . " " . $1;
-        } elsif ($opt =~ /^--prefix=(.*)/i ) {
-            $prefix = $1;
+    my @buildArgs = @ARGV;
+    if ($noWebKit1) {
+        unshift(@buildArgs, "--disable-webkit1");
+    }
+    if ($noWebKit2) {
+        unshift(@buildArgs, "--disable-webkit2");
+    }
+
+    # Configurable features listed here should be kept in sync with the
+    # features for which there exists a configuration option in configure.ac.
+    my %configurableFeatures = (
+        "gamepad" => 1,
+        "geolocation" => 1,
+        "media-stream" => 1,
+        "svg" => 1,
+        "svg-fonts" => 1,
+        "video" => 1,
+        "webgl" => 1,
+        "web-audio" => 1,
+        "xslt" => 1,
+    );
+    my @overridableFeatures = ();
+    foreach (@features) {
+        if ($configurableFeatures{$_->{option}}) {
+            push @buildArgs, autotoolsFlag(${$_->{value}}, $_->{option});;
         } else {
-            push @buildArgs, $opt;
+            push @overridableFeatures, $_->{define} . "=" . (${$_->{value}} ? "1" : "0");
         }
     }
 
+    $makeArgs = $makeArgs || "";
+    $makeArgs = $makeArgs . " " . $ENV{"WebKitMakeArguments"} if $ENV{"WebKitMakeArguments"};
+
     # Automatically determine the number of CPUs for make only
     # if make arguments haven't already been specified.
     if ($makeArgs eq "") {
@@ -2034,9 +2119,6 @@ sub buildAutotoolsProject($@)
         push @buildArgs, "--disable-debug";
     }
 
-    # Enable unstable features when building through build-webkit.
-    push @buildArgs, "--enable-unstable-features";
-
     if (checkForArgumentAndRemoveFromArrayRef("--update-gtk", \@buildArgs)) {
         # Force autogen to run, to catch the possibly updated libraries.
         system("rm -f previous-autogen-arguments.txt");
@@ -2047,9 +2129,10 @@ sub buildAutotoolsProject($@)
     # If GNUmakefile exists, don't run autogen.sh unless its arguments
     # have changed. The makefile should be smart enough to track autotools
     # dependencies and re-run autogen.sh when build files change.
-    runAutogenForAutotoolsProjectIfNecessary($dir, $prefix, $sourceDir, $project, @buildArgs);
+    my $joinedOverridableFeatures = join(" ", @overridableFeatures);
+    runAutogenForAutotoolsProjectIfNecessary($dir, $prefix, $sourceDir, $project, $joinedOverridableFeatures, @buildArgs);
 
-    my $runWithJhbuild = jhbuildWrapperPrefixIfNeeded();
+    my $runWithJhbuild = join(" ", jhbuildWrapperPrefixIfNeeded());
     if (system("$runWithJhbuild $make $makeArgs") ne 0) {
         die "\nFailed to build WebKit using '$make'!\n";
     }
@@ -2057,9 +2140,11 @@ sub buildAutotoolsProject($@)
     chdir ".." or die;
 
     if ($project eq 'WebKit' && !isCrossCompilation()) {
-        my @docGenerationOptions = ($runWithJhbuild, "$sourceDir/Tools/gtk/generate-gtkdoc", "--skip-html");
+        my @docGenerationOptions = ("$sourceDir/Tools/gtk/generate-gtkdoc", "--skip-html");
         push(@docGenerationOptions, productDir());
 
+        unshift(@docGenerationOptions, jhbuildWrapperPrefixIfNeeded());
+
         if (system(@docGenerationOptions)) {
             die "\n gtkdoc did not build without warnings\n";
         }
@@ -2071,14 +2156,18 @@ sub buildAutotoolsProject($@)
 sub jhbuildWrapperPrefixIfNeeded()
 {
     if (-e getJhbuildPath()) {
+        my @prefix = (File::Spec->catfile(sourceDir(), "Tools", "jhbuild", "jhbuild-wrapper"));
         if (isEfl()) {
-            return File::Spec->catfile(sourceDir(), "Tools", "efl", "run-with-jhbuild");
+            push(@prefix, "--efl");
         } elsif (isGtk()) {
-            return File::Spec->catfile(sourceDir(), "Tools", "gtk", "run-with-jhbuild");
+            push(@prefix, "--gtk");
         }
+        push(@prefix, "run");
+
+        return @prefix;
     }
 
-    return "";
+    return ();
 }
 
 sub removeCMakeCache()
@@ -2119,7 +2208,7 @@ sub generateBuildSystemFromCMakeProject
 
     # We call system("cmake @args") instead of system("cmake", @args) so that @args is
     # parsed for shell metacharacters.
-    my $wrapper = jhbuildWrapperPrefixIfNeeded() . " ";
+    my $wrapper = join(" ", jhbuildWrapperPrefixIfNeeded()) . " ";
     my $returnCode = system($wrapper . "cmake @args");
 
     chdir($originalWorkingDirectory);
@@ -2139,7 +2228,7 @@ sub buildCMakeGeneratedProject($)
 
     # We call system("cmake @args") instead of system("cmake", @args) so that @args is
     # parsed for shell metacharacters. In particular, $makeArgs may contain such metacharacters.
-    my $wrapper = jhbuildWrapperPrefixIfNeeded() . " ";
+    my $wrapper = join(" ", jhbuildWrapperPrefixIfNeeded()) . " ";
     return system($wrapper . "cmake @args");
 }
 
@@ -2167,6 +2256,9 @@ sub buildCMakeProjectOrExit($$$$@)
 
     $returnCode = exitStatus(generateBuildSystemFromCMakeProject($port, $prefixPath, @cmakeArgs));
     exit($returnCode) if $returnCode;
+    if (isBlackBerry()) {
+        return 0 if (defined($ENV{"GENERATE_CMAKE_PROJECT_ONLY"}) eq '1');
+    }
     $returnCode = exitStatus(buildCMakeGeneratedProject($makeArgs));
     exit($returnCode) if $returnCode;
     return 0;
@@ -2175,7 +2267,7 @@ sub buildCMakeProjectOrExit($$$$@)
 sub cmakeBasedPortArguments()
 {
     return blackberryCMakeArguments() if isBlackBerry();
-    return ('-DCMAKE_WINCE_SDK="STANDARDSDK_500 (ARMV4I)"') if isWinCE();
+    return ('-G "Visual Studio 8 2005 STANDARDSDK_500 (ARMV4I)"') if isWinCE();
     return ();
 }
 
@@ -2205,6 +2297,7 @@ sub buildQMakeProjects
 
     my $make = qtMakeCommand($qmakebin);
     my $makeargs = "";
+    my $command;
     my $installHeaders;
     my $installLibs;
     for my $i (0 .. $#buildParams) {
@@ -2229,14 +2322,23 @@ sub buildQMakeProjects
         $makeargs .= " -j" . numberOfCPUs();
     }
 
-    my $qmakepath = File::Spec->catfile(sourceDir(), "Tools", "qmake");
-    my $qmakecommand;
-    if (isWindows()) {
-        $qmakecommand = "(set QMAKEPATH=$qmakepath) && $qmakebin";
-    } else {
-        $qmakecommand = "QMAKEPATH=$qmakepath $qmakebin";
+    $make = "$make $makeargs";
+    $make =~ s/\s+$//;
+
+    my $originalCwd = getcwd();
+    my $dir = File::Spec->canonpath(productDir());
+    File::Path::mkpath($dir);
+    chdir $dir or die "Failed to cd into " . $dir . "\n";
+
+    if ($clean) {
+        $command = "$make distclean";
+        print "\nCalling '$command' in " . $dir . "\n\n";
+        return system $command;
     }
 
+    my $qmakepath = File::Spec->catfile(sourceDir(), "Tools", "qmake");
+    my $qmakecommand = $qmakebin;
+
     my $config = configuration();
     push @buildArgs, "INSTALL_HEADERS=" . $installHeaders if defined($installHeaders);
     push @buildArgs, "INSTALL_LIBS=" . $installLibs if defined($installLibs);
@@ -2245,153 +2347,105 @@ sub buildQMakeProjects
     if ($passedConfig =~ m/debug/i) {
         push @buildArgs, "CONFIG-=release";
         push @buildArgs, "CONFIG+=debug";
-    } elsif (!$passedConfig or $passedConfig =~ m/release/i) {
+    } elsif ($passedConfig =~ m/release/i) {
         push @buildArgs, "CONFIG+=release";
         push @buildArgs, "CONFIG-=debug";
-    } else {
+    } elsif ($passedConfig) {
         die "Build type $passedConfig is not supported with --qt.\n";
     }
-    push @buildArgs, "CONFIG-=debug_and_release" if ($passedConfig && isDarwin());
 
-    my $originalCwd = getcwd();
-    my $dir = File::Spec->canonpath(productDir());
-    File::Path::mkpath($dir);
-    chdir $dir or die "Failed to cd into " . $dir . "\n";
-
-    my %defines = qtFeatureDefaults(\@buildArgs, \$qconfigs);
+    # Using build-webkit to build assumes you want a developer-build
+    push @buildArgs, "CONFIG-=production_build";
 
     my $svnRevision = currentSVNRevision();
+    my $previousSvnRevision = "unknown";
 
     my $buildHint = "";
 
-    my $pathToDefinesCache = File::Spec->catfile($dir, ".webkit.config");
-    my $pathToOldDefinesFile = File::Spec->catfile($dir, "defaults.txt");
-
-    # FIXME: Get rid of .webkit.config and defaults.txt and move all the logic to .qmake.cache
-
-    # Ease transition to new build layout
-    if (-e $pathToOldDefinesFile) {
-        print "Old build layout detected";
-        $buildHint = "clean";
-    } elsif (-e $pathToDefinesCache && open(DEFAULTS, $pathToDefinesCache)) {
-        my %previousDefines;
-        while (<DEFAULTS>) {
-            if ($_ =~ m/(\S+)=(\S+)/gi) {
-                $previousDefines{$1} = $2;
+    my $pathToBuiltRevisions = File::Spec->catfile($dir, ".builtRevisions.cache");
+    if (-e $pathToBuiltRevisions && open(BUILTREVISIONS, $pathToBuiltRevisions)) {
+        while (<BUILTREVISIONS>) {
+            if ($_ =~ m/^SVN_REVISION\s=\s(\d+)$/) {
+                $previousSvnRevision = $1;
             }
         }
-        close (DEFAULTS);
-
-        $previousDefines{"SVN_REVISION"} = "unknown" if not exists $previousDefines{"SVN_REVISION"};
-
-        if ($svnRevision ne $previousDefines{"SVN_REVISION"}) {
-            print "Last built revision was " . $previousDefines{"SVN_REVISION"} .
-                ", now at revision $svnRevision. Full incremental build needed.\n";
-
-            $buildHint = "incremental";
-        }
-
-        # Don't confuse the should-we-clean heuristics below
-        delete($previousDefines{"SVN_REVISION"});
-
-        my @uniqueDefineNames = keys %{ +{ map { $_, 1 } (keys %defines, keys %previousDefines) } };
-        foreach my $define (@uniqueDefineNames) {
-            if (! exists $previousDefines{$define}) {
-                print "Feature $define added";
-                $buildHint = "clean";
-                last;
-            }
-
-            if (! exists $defines{$define}) {
-                print "Feature $define removed";
-                $buildHint = "clean";
-                last;
-            }
-
-            if ($defines{$define} != $previousDefines{$define}) {
-                print "Feature $define changed ($previousDefines{$define} -> $defines{$define})";
-                $buildHint = "clean";
-                last;
-            }
-        }
-    } else {
-        # Missing build cache suggests we had a broken build after a clean,
-        # so we assume we have to do an incremental build just in case.
-        $buildHint = "incremental";
+        close(BUILTREVISIONS);
     }
 
-    if ($buildHint eq "clean") {
-        print ", clean build needed!\n";
-        # FIXME: This STDIN/STDOUT check does not work on the bots. Disable until it does.
-        # if (! -t STDIN || ( &promptUser("Would you like to clean the build directory?", "yes") eq "yes")) {
-            chdir $originalCwd;
-            File::Path::rmtree($dir);
-            File::Path::mkpath($dir);
-            chdir $dir or die "Failed to cd into " . $dir . "\n";
-        #}
-
-        # Still trigger an incremental build
-        $buildHint = "incremental";
-    }
-
-    if ($buildHint eq "incremental") {
-        my $qmakeDefines = "DEFINES +=";
-        foreach my $key (sort keys %defines) {
-            $qmakeDefines .= " \\\n    $key=$defines{$key}";
-        }
-        open(QMAKE_CACHE, ">.qmake.cache") or die "Cannot create .qmake.cache!\n";
-        print QMAKE_CACHE "CONFIG += webkit_configured $qconfigs\n";
-        print QMAKE_CACHE $qmakeDefines."\n";
-        close(QMAKE_CACHE);
-    }
-
-    # Save config up-front so we can detect changes to the build config even
-    # when the user re-configures after aborting the build.
-    open(DEFAULTS, ">$pathToDefinesCache");
-    print DEFAULTS "# These defines were set when building WebKit last time\n";
-    foreach my $key (sort keys %defines) {
-        print DEFAULTS "$key=$defines{$key}\n";
-    }
-    close(DEFAULTS);
-
     my $result = 0;
 
-    my $makefile = File::Spec->catfile($dir, "Makefile");
-    if (! -e $makefile) {
-        push @buildArgs, "-after OVERRIDE_SUBDIRS=\"@{$projects}\"" if @{$projects};
+    # Run qmake, regadless of having a makefile or not, so that qmake can
+    # detect changes to the configuration.
 
-        push @buildArgs, File::Spec->catfile(sourceDir(), "WebKit.pro");
-        my $command = "$qmakecommand @buildArgs";
-        print "Calling '$command' in " . $dir . "\n\n";
-        print "Installation headers directory: $installHeaders\n" if(defined($installHeaders));
-        print "Installation libraries directory: $installLibs\n" if(defined($installLibs));
+    push @buildArgs, "-after OVERRIDE_SUBDIRS=\"@{$projects}\"" if @{$projects};
+    unshift @buildArgs, File::Spec->catfile(sourceDir(), "WebKit.pro");
+    $command = "$qmakecommand @buildArgs";
+    print "Calling '$command' in " . $dir . "\n\n";
+    print "Installation headers directory: $installHeaders\n" if(defined($installHeaders));
+    print "Installation libraries directory: $installLibs\n" if(defined($installLibs));
 
-        $result = system "$command";
-        if ($result ne 0) {
-           die "Failed to setup build environment using $qmakebin!\n";
+    my $configChanged = 0;
+    open(QMAKE, "$command 2>&1 |") || die "Could not execute qmake";
+    while (<QMAKE>) {
+        $configChanged = 1 if $_ =~ m/The configuration was changed since the last build/;
+        print $_;
+    }
+
+    close(QMAKE);
+    $result = $?;
+
+    if ($result ne 0) {
+       die "\nFailed to set up build environment using $qmakebin!\n";
+    }
+
+    my $maybeNeedsCleanBuild = 0;
+    my $needsIncrementalBuild = 0;
+
+    # Full incremental build (run qmake) needed on buildbots and EWS bots always.
+    if (grep(/CONFIG\+=buildbot/,@buildParams)) {
+        $needsIncrementalBuild = 1;
+    }
+
+    if ($svnRevision ne $previousSvnRevision) {
+        print "Last built revision was " . $previousSvnRevision .
+            ", now at revision $svnRevision. Full incremental build needed.\n";
+        $needsIncrementalBuild = 1;
+
+        my @fileList = listOfChangedFilesBetweenRevisions(sourceDir(), $previousSvnRevision, $svnRevision);
+
+        foreach (@fileList) {
+            if (m/\.pr[oif]$/ or
+                m/\.qmake.conf$/ or
+                m/^Tools\/qmake\//
+               ) {
+                print "Change to $_ detected, clean build may be needed.\n";
+                $maybeNeedsCleanBuild = 1;
+                last;
+            }
         }
     }
 
-    my $command = "$make $makeargs";
-    $command =~ s/\s+$//;
-
-    if ($clean) {
-        $command = "$command distclean";
-    } elsif ($buildHint eq "incremental") {
-        $command = "$command incremental";
+    if ($configChanged) {
+        print "Calling '$make wipeclean' in " . $dir . "\n\n";
+        $result = system "$make wipeclean";
     }
 
-    print "Calling '$command' in " . $dir . "\n\n";
+    $command = "$make";
+    if ($needsIncrementalBuild) {
+        $command .= " incremental";
+    }
+
+    print "\nCalling '$command' in " . $dir . "\n\n";
     $result = system $command;
 
     chdir ".." or die;
 
     if ($result eq 0) {
         # Now that the build completed successfully we can save the SVN revision
-        open(DEFAULTS, ">>$pathToDefinesCache");
-        print DEFAULTS "SVN_REVISION=$svnRevision\n";
-        close(DEFAULTS);
-    } elsif ($buildHint eq "" && exitStatus($result)) {
+        open(BUILTREVISIONS, ">>$pathToBuiltRevisions");
+        print BUILTREVISIONS "SVN_REVISION = $svnRevision\n";
+        close(BUILTREVISIONS);
+    } elsif (!$command =~ /incremental/ && exitStatus($result)) {
         my $exitCode = exitStatus($result);
         my $failMessage = <<EOF;
 
@@ -2404,16 +2458,24 @@ The build failed with exit code $exitCode. This may have been because you
   - added a new resource to a qrc file
 
 as dependencies are not automatically re-computed for local developer builds.
-You may try computing dependencies manually by running 'make qmake' in:
+You may try computing dependencies manually by running 'make qmake_all' in:
 
   $dir
 
-or passing --makeargs="qmake" to build-webkit.
+or passing --makeargs="qmake_all" to build-webkit.
 
 =========================
 
 EOF
         print "$failMessage";
+    } elsif ($maybeNeedsCleanBuild) {
+        print "\nIncremental build failed, clean build needed. \n";
+        print "Calling '$make wipeclean' in " . $dir . "\n\n";
+        chdir $dir or die;
+        system "$make wipeclean";
+
+        print "\nCalling '$make' in " . $dir . "\n\n";
+        $result = system $make;
     }
 
     return $result;
@@ -2421,13 +2483,13 @@ EOF
 
 sub buildGtkProject
 {
-    my ($project, $clean, @buildArgs) = @_;
+    my ($project, $clean, $prefix, $makeArgs, $noWebKit1, $noWebKit2, @features) = @_;
 
     if ($project ne "WebKit" and $project ne "JavaScriptCore" and $project ne "WTF") {
         die "Unsupported project: $project. Supported projects: WebKit, JavaScriptCore, WTF\n";
     }
 
-    return buildAutotoolsProject($project, $clean, @buildArgs);
+    return buildAutotoolsProject($project, $clean, $prefix, $makeArgs, $noWebKit1, $noWebKit2, @features);
 }
 
 sub buildChromiumMakefile($$@)
@@ -2460,7 +2522,17 @@ sub buildChromiumNinja($$@)
     }
     my $command = "";
 
-    $command .= "ninja -C out/$config $target $makeArgs";
+    # Find ninja.
+    my $ninjaPath;
+    if (commandExists('ninja')) {
+        $ninjaPath = 'ninja';
+    } elsif (-e 'Source/WebKit/chromium/depot_tools/ninja') {
+        $ninjaPath = 'Source/WebKit/chromium/depot_tools/ninja';
+    } else {
+        die "ninja not found. Install chromium's depot_tools by running update-webkit first\n";
+    }
+
+    $command .= "$ninjaPath -C out/$config $target $makeArgs";
 
     print "$command\n";
     return system $command;
@@ -2522,10 +2594,10 @@ sub buildChromium($@)
     if (isDarwin() && !isChromiumAndroid() && !isChromiumMacMake() && !isChromiumNinja()) {
         # Mac build - builds the root xcode project.
         $result = buildXCodeProject("Source/WebKit/chromium/All", $clean, "-configuration", configuration(), @options);
-    } elsif (isCygwin() || isWindows()) {
+    } elsif ((isCygwin() || isWindows()) && !isChromiumNinja()) {
         # Windows build - builds the root visual studio solution.
         $result = buildChromiumVisualStudioProject("Source/WebKit/chromium/All.sln", $clean);
-    } elsif (isChromiumNinja() && !isChromiumAndroid()) {
+    } elsif (isChromiumNinja()) {
         $result = buildChromiumNinja("all", $clean, @options);
     } elsif (isLinux() || isChromiumAndroid() || isChromiumMacMake()) {
         # Linux build - build using make.
@@ -2570,7 +2642,7 @@ sub printHelpAndExitForRunAndDebugWebKitAppIfNeeded
     print STDERR <<EOF;
 Usage: @{[basename($0)]} [options] [args ...]
   --help                            Show this help message
-  --no-saved-state                  Disable application resume for the session on Mac OS 10.7
+  --no-saved-state                  Launch the application without state restoration (OS X 10.7 and later)
   --guard-malloc                    Enable Guard Malloc (OS X only)
   --use-web-process-xpc-service     Launch the Web Process as an XPC Service (OS X only)
 EOF
@@ -2578,7 +2650,8 @@ EOF
     if ($includeOptionsForDebugging) {
         print STDERR <<EOF;
   --target-web-process              Debug the web process
-  --use-lldb                        Use LLDB
+  --use-gdb                         Use GDB (this is the default when using Xcode 4.4 or earlier)
+  --use-lldb                        Use LLDB (this is the default when using Xcode 4.5 or later)
 EOF
     }
 
@@ -2587,8 +2660,11 @@ EOF
 
 sub argumentsForRunAndDebugMacWebKitApp()
 {
-    my @args = @ARGV;
+    my @args = ();
     push @args, ("-ApplePersistenceIgnoreState", "YES") if !isSnowLeopard() && checkForArgumentAndRemoveFromArrayRef("--no-saved-state", \@args);
+    push @args, ("-WebKit2UseXPCServiceForWebProcess", "YES") if shouldUseXPCServiceForWebProcess();
+    unshift @args, @ARGV;
+
     return @args;
 }
 
@@ -2602,12 +2678,6 @@ sub runMacWebKitApp($;$)
 
     setUpGuardMallocIfNeeded();
 
-    if (shouldUseXPCServiceForWebProcess()) {
-        $ENV{__XPC_DYLD_FRAMEWORK_PATH} = $productDir;
-        appendToEnvironmentVariableList("__XPC_DYLD_INSERT_LIBRARIES", File::Spec->catfile($productDir, "WebProcessShim.dylib"));
-        $ENV{WEBKIT_USE_XPC_SERVICE_FOR_WEB_PROCESS} = "YES";
-    }
-
     if (defined($useOpenCommand) && $useOpenCommand == USE_OPEN_COMMAND) {
         return system("open", "-W", "-a", $appPath, "--args", argumentsForRunAndDebugMacWebKitApp());
     }
@@ -2645,11 +2715,6 @@ sub execMacWebKitAppForDebugging($)
 
     my @architectureFlags = ($architectureSwitch, architecture());
     if (!shouldTargetWebProcess()) {
-        if (shouldUseXPCServiceForWebProcess()) {
-            $ENV{__XPC_DYLD_FRAMEWORK_PATH} = $productDir;
-            appendToEnvironmentVariableList("__XPC_DYLD_INSERT_LIBRARIES", File::Spec->catfile($productDir, "WebProcessShim.dylib"));
-            $ENV{WEBKIT_USE_XPC_SERVICE_FOR_WEB_PROCESS} = "YES";
-        }
         print "Starting @{[basename($appPath)]} under $debugger with DYLD_FRAMEWORK_PATH set to point to built WebKit in $productDir.\n";
         exec { $debuggerPath } $debuggerPath, @architectureFlags, $argumentsSeparator, $appPath, argumentsForRunAndDebugMacWebKitApp() or die;
     } else {
@@ -2657,7 +2722,7 @@ sub execMacWebKitAppForDebugging($)
             die "Targetting the Web Process is not compatible with using an XPC Service for the Web Process at this time.";
         }
         
-        my $webProcessShimPath = File::Spec->catfile($productDir, "WebProcessShim.dylib");
+        my $webProcessShimPath = File::Spec->catfile($productDir, "SecItemShim.dylib");
         my $webProcessPath = File::Spec->catdir($productDir, "WebProcess.app");
         my $webKit2ExecutablePath = File::Spec->catfile($productDir, "WebKit2.framework", "WebKit2");
 
