@@ -29,7 +29,13 @@
 #include "PlatformWebView.h"
 #include "StringFunctions.h"
 #include "TestInvocation.h"
+#include <WebKit2/WKAuthenticationChallenge.h>
+#include <WebKit2/WKAuthenticationDecisionListener.h>
 #include <WebKit2/WKContextPrivate.h>
+#include <WebKit2/WKCredential.h>
+#include <WebKit2/WKNotification.h>
+#include <WebKit2/WKNotificationManager.h>
+#include <WebKit2/WKNotificationPermissionRequest.h>
 #include <WebKit2/WKNumber.h>
 #include <WebKit2/WKPageGroup.h>
 #include <WebKit2/WKPagePrivate.h>
@@ -41,6 +47,7 @@
 #include <stdlib.h>
 #include <string>
 #include <wtf/PassOwnPtr.h>
+#include <wtf/text/CString.h>
 
 #if PLATFORM(MAC)
 #include <WebKit2/WKPagePrivateMac.h>
@@ -50,12 +57,16 @@
 #include "EventSenderProxy.h"
 #endif
 
+#if !PLATFORM(MAC)
+#include <WebKit2/WKTextChecker.h>
+#endif
+
 namespace WTR {
 
-// defaultLongTimeout + defaultShortTimeout should be less than 50,
+// defaultLongTimeout + defaultShortTimeout should be less than 80,
 // the default timeout value of the test harness so we can detect an
 // unresponsive web process.
-static const double defaultLongTimeout = 30;
+static const double defaultLongTimeout = 60;
 static const double defaultShortTimeout = 15;
 static const double defaultNoTimeout = -1;
 
@@ -78,6 +89,7 @@ TestController::TestController(int argc, const char* argv[])
     , m_printSeparators(false)
     , m_usingServerMode(false)
     , m_gcBetweenTests(false)
+    , m_shouldDumpPixelsForAllTests(false)
     , m_state(Initial)
     , m_doneResetting(false)
     , m_longTimeout(defaultLongTimeout)
@@ -85,12 +97,14 @@ TestController::TestController(int argc, const char* argv[])
     , m_noTimeout(defaultNoTimeout)
     , m_useWaitToDumpWatchdogTimer(true)
     , m_forceNoTimeout(false)
+    , m_timeout(0)
     , m_didPrintWebProcessCrashedMessage(false)
     , m_shouldExitWhenWebProcessCrashes(true)
     , m_beforeUnloadReturnValue(true)
-#if PLATFORM(MAC) || PLATFORM(QT) || PLATFORM(GTK) || PLATFORM(EFL)
-    , m_eventSenderProxy(new EventSenderProxy(this))
-#endif
+    , m_isGeolocationPermissionSet(false)
+    , m_isGeolocationPermissionAllowed(false)
+    , m_policyDelegateEnabled(false)
+    , m_policyDelegatePermissive(false)
 {
     initialize(argc, argv);
     controller = this;
@@ -100,37 +114,25 @@ TestController::TestController(int argc, const char* argv[])
 
 TestController::~TestController()
 {
+    platformDestroy();
 }
 
-static WKRect getWindowFrameMainPage(WKPageRef page, const void* clientInfo)
-{
-    PlatformWebView* view = static_cast<TestController*>(const_cast<void*>(clientInfo))->mainWebView();
-    return view->windowFrame();
-}
-
-static void setWindowFrameMainPage(WKPageRef page, WKRect frame, const void* clientInfo)
-{
-    PlatformWebView* view = static_cast<TestController*>(const_cast<void*>(clientInfo))->mainWebView();
-    view->setWindowFrame(frame);
-}
-
-static WKRect getWindowFrameOtherPage(WKPageRef page, const void* clientInfo)
+static WKRect getWindowFrame(WKPageRef page, const void* clientInfo)
 {
     PlatformWebView* view = static_cast<PlatformWebView*>(const_cast<void*>(clientInfo));
     return view->windowFrame();
 }
 
-static void setWindowFrameOtherPage(WKPageRef page, WKRect frame, const void* clientInfo)
+static void setWindowFrame(WKPageRef page, WKRect frame, const void* clientInfo)
 {
     PlatformWebView* view = static_cast<PlatformWebView*>(const_cast<void*>(clientInfo));
     view->setWindowFrame(frame);
 }
 
-static bool runBeforeUnloadConfirmPanel(WKPageRef page, WKStringRef message, WKFrameRef frame, const void *clientInfo)
+static bool runBeforeUnloadConfirmPanel(WKPageRef page, WKStringRef message, WKFrameRef frame, const void*)
 {
-    TestController* testController = static_cast<TestController*>(const_cast<void*>(clientInfo));
     printf("CONFIRM NAVIGATION: %s\n", toSTD(message).c_str());
-    return testController->beforeUnloadReturnValue();
+    return TestController::shared().beforeUnloadReturnValue();
 }
 
 static unsigned long long exceededDatabaseQuota(WKPageRef, WKFrameRef, WKSecurityOriginRef, WKStringRef, WKStringRef, unsigned long long, unsigned long long, unsigned long long, unsigned long long, const void*)
@@ -158,6 +160,7 @@ static void closeOtherPage(WKPageRef page, const void* clientInfo)
 static void focus(WKPageRef page, const void* clientInfo)
 {
     PlatformWebView* view = static_cast<PlatformWebView*>(const_cast<void*>(clientInfo));
+    view->focus();
     view->setWindowIsKey(true);
 }
 
@@ -167,6 +170,16 @@ static void unfocus(WKPageRef page, const void* clientInfo)
     view->setWindowIsKey(false);
 }
 
+static void decidePolicyForGeolocationPermissionRequest(WKPageRef, WKFrameRef, WKSecurityOriginRef, WKGeolocationPermissionRequestRef permissionRequest, const void* clientInfo)
+{
+    TestController::shared().handleGeolocationPermissionRequest(permissionRequest);
+}
+
+int TestController::getCustomTimeout()
+{
+    return m_timeout;
+}
+
 WKPageRef TestController::createOtherPage(WKPageRef oldPage, WKURLRequestRef, WKDictionaryRef, WKEventModifiers, WKEventMouseButton, const void*)
 {
     PlatformWebView* view = new PlatformWebView(WKPageGetContext(oldPage), WKPageGetPageGroup(oldPage));
@@ -199,14 +212,14 @@ WKPageRef TestController::createOtherPage(WKPageRef oldPage, WKURLRequestRef, WK
         0, // setStatusBarIsVisible
         0, // isResizable
         0, // setIsResizable
-        getWindowFrameOtherPage,
-        setWindowFrameOtherPage,
+        getWindowFrame,
+        setWindowFrame,
         runBeforeUnloadConfirmPanel,
         0, // didDraw
         0, // pageDidScroll
         exceededDatabaseQuota,
         0, // runOpenPanel
-        0, // decidePolicyForGeolocationPermissionRequest
+        decidePolicyForGeolocationPermissionRequest,
         0, // headerHeight
         0, // footerHeight
         0, // drawHeader
@@ -220,6 +233,9 @@ WKPageRef TestController::createOtherPage(WKPageRef oldPage, WKURLRequestRef, WK
         0, // mouseDidMoveOverElement
         0, // decidePolicyForNotificationPermissionRequest
         0, // unavailablePluginButtonClicked
+        0, // showColorPicker
+        0, // hideColorPicker
+        0, // shouldInstantiatePlugin
     };
     WKPageSetPageUIClient(newPage, &otherPageUIClient);
 
@@ -281,11 +297,16 @@ void TestController::initialize(int argc, const char* argv[])
             m_gcBetweenTests = true;
             continue;
         }
+        if (argument == "--pixel-tests" || argument == "-p") {
+            m_shouldDumpPixelsForAllTests = true;
+            continue;
+        }
         if (argument == "--print-supported-features") {
             printSupportedFeatures = true;
             break;
         }
 
+
         // Skip any other arguments that begin with '--'.
         if (argument.length() >= 2 && argument[0] == '-' && argument[1] == '-')
             continue;
@@ -313,13 +334,24 @@ void TestController::initialize(int argc, const char* argv[])
     m_pageGroup.adopt(WKPageGroupCreateWithIdentifier(pageGroupIdentifier.get()));
 
     m_context.adopt(WKContextCreateWithInjectedBundlePath(injectedBundlePath()));
+    m_geolocationProvider = adoptPtr(new GeolocationProviderMock(m_context.get()));
 
-    const char* path = libraryPathForTesting();
-    if (path) {
-        Vector<char> databaseDirectory(strlen(path) + strlen("/Databases") + 1);
-        sprintf(databaseDirectory.data(), "%s%s", path, "/Databases");
-        WKRetainPtr<WKStringRef> databaseDirectoryWK(AdoptWK, WKStringCreateWithUTF8CString(databaseDirectory.data()));
-        WKContextSetDatabaseDirectory(m_context.get(), databaseDirectoryWK.get());
+    if (const char* dumpRenderTreeTemp = libraryPathForTesting()) {
+        WKRetainPtr<WKStringRef> dumpRenderTreeTempWK(AdoptWK, WKStringCreateWithUTF8CString(dumpRenderTreeTemp));
+        WKContextSetDatabaseDirectory(m_context.get(), dumpRenderTreeTempWK.get());
+        WKContextSetLocalStorageDirectory(m_context.get(), dumpRenderTreeTempWK.get());
+        WKContextSetDiskCacheDirectory(m_context.get(), dumpRenderTreeTempWK.get());
+        WKContextSetCookieStorageDirectory(m_context.get(), dumpRenderTreeTempWK.get());
+
+        // WebCore::pathByAppendingComponent is not used here because of the namespace,
+        // which leads us to this ugly #ifdef and file path concatenation.
+#if OS(WINDOWS)
+        const char separator = '\\';
+#else
+        const char separator = '/';
+#endif
+        String iconDatabaseFileTemp = String::fromUTF8(dumpRenderTreeTemp) + separator + String(ASCIILiteral("WebpageIcons.db"));
+        WKContextSetIconDatabasePath(m_context.get(), toWK(iconDatabaseFileTemp).get());
     }
 
     platformInitializeContext();
@@ -333,20 +365,28 @@ void TestController::initialize(int argc, const char* argv[])
     };
     WKContextSetInjectedBundleClient(m_context.get(), &injectedBundleClient);
 
+    WKNotificationManagerRef notificationManager = WKContextGetNotificationManager(m_context.get());
+    WKNotificationProvider notificationKit = m_webNotificationProvider.provider();
+    WKNotificationManagerSetProvider(notificationManager, &notificationKit);
+
     if (testPluginDirectory())
         WKContextSetAdditionalPluginsDirectory(m_context.get(), testPluginDirectory());
 
-    m_mainWebView = adoptPtr(new PlatformWebView(m_context.get(), m_pageGroup.get()));
+    createWebViewWithOptions(0);
+}
 
+void TestController::createWebViewWithOptions(WKDictionaryRef options)
+{
+    m_mainWebView = adoptPtr(new PlatformWebView(m_context.get(), m_pageGroup.get(), options));
     WKPageUIClient pageUIClient = {
         kWKPageUIClientCurrentVersion,
-        this,
+        m_mainWebView.get(),
         0, // createNewPage_deprecatedForUseWithV0
         0, // showPage
         0, // close
         0, // takeFocus
-        0, // focus
-        0, // unfocus
+        focus,
+        unfocus,
         0, // runJavaScriptAlert
         0, // runJavaScriptConfirm
         0, // runJavaScriptPrompt
@@ -363,14 +403,14 @@ void TestController::initialize(int argc, const char* argv[])
         0, // setStatusBarIsVisible
         0, // isResizable
         0, // setIsResizable
-        getWindowFrameMainPage,
-        setWindowFrameMainPage,
+        getWindowFrame,
+        setWindowFrame,
         runBeforeUnloadConfirmPanel,
         0, // didDraw
         0, // pageDidScroll
         exceededDatabaseQuota,
         0, // runOpenPanel
-        0, // decidePolicyForGeolocationPermissionRequest
+        decidePolicyForGeolocationPermissionRequest,
         0, // headerHeight
         0, // footerHeight
         0, // drawHeader
@@ -382,8 +422,11 @@ void TestController::initialize(int argc, const char* argv[])
         0, // shouldInterruptJavaScript
         createOtherPage,
         0, // mouseDidMoveOverElement
-        0, // decidePolicyForNotificationPermissionRequest
-        0, // unavailablePluginButtonClicked
+        decidePolicyForNotificationPermissionRequest, // decidePolicyForNotificationPermissionRequest
+        unavailablePluginButtonClicked,
+        0, // showColorPicker
+        0, // hideColorPicker
+        0, // shouldInstantiatePlugin
     };
     WKPageSetPageUIClient(m_mainWebView->page(), &pageUIClient);
 
@@ -405,7 +448,7 @@ void TestController::initialize(int argc, const char* argv[])
         0, // didFailToInitializePlugin
         0, // didDisplayInsecureContentForFrame
         0, // canAuthenticateAgainstProtectionSpaceInFrame
-        0, // didReceiveAuthenticationChallengeInFrame
+        didReceiveAuthenticationChallengeInFrame, // didReceiveAuthenticationChallengeInFrame
         0, // didStartProgress
         0, // didChangeProgress
         0, // didFinishProgress
@@ -422,8 +465,34 @@ void TestController::initialize(int argc, const char* argv[])
         0, // pluginDidFail
         0, // didReceiveIntentForFrame
         0, // registerIntentServiceForFrame
+        0, // didLayout
     };
     WKPageSetPageLoaderClient(m_mainWebView->page(), &pageLoaderClient);
+
+    WKPagePolicyClient pagePolicyClient = {
+        kWKPagePolicyClientCurrentVersion,
+        this,
+        decidePolicyForNavigationAction,
+        0, // decidePolicyForNewWindowAction
+        decidePolicyForResponse,
+        0, // unableToImplementPolicy
+    };
+    WKPageSetPagePolicyClient(m_mainWebView->page(), &pagePolicyClient);
+}
+
+void TestController::ensureViewSupportsOptions(WKDictionaryRef options)
+{
+    if (m_mainWebView && !m_mainWebView->viewSupportsOptions(options)) {
+        WKPageSetPageUIClient(m_mainWebView->page(), 0);
+        WKPageSetPageLoaderClient(m_mainWebView->page(), 0);
+        WKPageSetPagePolicyClient(m_mainWebView->page(), 0);
+        WKPageClose(m_mainWebView->page());
+        
+        m_mainWebView = nullptr;
+
+        createWebViewWithOptions(options);
+        resetStateToConsistentValues();
+    }
 }
 
 bool TestController::resetStateToConsistentValues()
@@ -447,6 +516,11 @@ bool TestController::resetStateToConsistentValues()
 
     // FIXME: This function should also ensure that there is only one page open.
 
+    // Reset the EventSender for each test.
+#if PLATFORM(MAC) || PLATFORM(QT) || PLATFORM(GTK) || PLATFORM(EFL)
+    m_eventSenderProxy = adoptPtr(new EventSenderProxy(this));
+#endif
+
     // Reset preferences
     WKPreferencesRef preferences = WKPageGroupGetPreferences(m_pageGroup.get());
     WKPreferencesResetTestRunnerOverrides(preferences);
@@ -468,12 +542,9 @@ bool TestController::resetStateToConsistentValues()
     WKPreferencesSetAsynchronousPluginInitializationEnabled(preferences, false);
     WKPreferencesSetAsynchronousPluginInitializationEnabledForAllPlugins(preferences, false);
     WKPreferencesSetArtificialPluginInitializationDelayEnabled(preferences, false);
-
-// [Qt][WK2]REGRESSION(r104881):It broke hundreds of tests
-// FIXME: https://bugs.webkit.org/show_bug.cgi?id=76247
-#if !PLATFORM(QT)
+    WKPreferencesSetTabToLinksEnabled(preferences, false);
+    WKPreferencesSetInteractiveFormValidationEnabled(preferences, true);
     WKPreferencesSetMockScrollbarsEnabled(preferences, true);
-#endif
 
 #if !PLATFORM(QT)
     static WKStringRef standardFontFamily = WKStringCreateWithUTF8CString("Times");
@@ -492,7 +563,11 @@ bool TestController::resetStateToConsistentValues()
     WKPreferencesSetSansSerifFontFamily(preferences, sansSerifFontFamily);
     WKPreferencesSetSerifFontFamily(preferences, serifFontFamily);
 #endif
+    WKPreferencesSetScreenFontSubstitutionEnabled(preferences, true);
     WKPreferencesSetInspectorUsesWebKitUserInterface(preferences, true);
+#if !PLATFORM(MAC)
+    WKTextCheckerContinuousSpellCheckingEnabledStateChanged(true);
+#endif
 
     // in the case that a test using the chrome input field failed, be sure to clean up for the next test
     m_mainWebView->removeChromeInputField();
@@ -501,6 +576,30 @@ bool TestController::resetStateToConsistentValues()
     // Re-set to the default backing scale factor by setting the custom scale factor to 0.
     WKPageSetCustomBackingScaleFactor(m_mainWebView->page(), 0);
 
+#if PLATFORM(EFL)
+    // EFL use a real window while other ports such as Qt don't.
+    // In EFL, we need to resize the window to the original size after calls to window.resizeTo.
+    WKRect rect = m_mainWebView->windowFrame();
+    m_mainWebView->setWindowFrame(WKRectMake(rect.origin.x, rect.origin.y, 800, 600));
+#endif
+
+    // Reset notification permissions
+    m_webNotificationProvider.reset();
+
+    // Reset Geolocation permissions.
+    m_geolocationPermissionRequests.clear();
+    m_isGeolocationPermissionSet = false;
+    m_isGeolocationPermissionAllowed = false;
+
+    // Reset Custom Policy Delegate.
+    setCustomPolicyDelegate(false, false);
+
+    m_workQueueManager.clearWorkQueue();
+
+    m_handlesAuthenticationChallenges = false;
+    m_authenticationUsername = String();
+    m_authenticationPassword = String();
+
     // Reset main page back to about:blank
     m_doneResetting = false;
 
@@ -510,11 +609,12 @@ bool TestController::resetStateToConsistentValues()
 }
 
 struct TestCommand {
-    TestCommand() : shouldDumpPixels(false) { }
+    TestCommand() : shouldDumpPixels(false), timeout(0) { }
 
     std::string pathOrURL;
     bool shouldDumpPixels;
     std::string expectedPixelHash;
+    int timeout;
 };
 
 class CommandTokenizer {
@@ -576,18 +676,20 @@ TestCommand parseInputLine(const std::string& inputLine)
     if (!tokenizer.hasNext())
         die(inputLine);
 
-    result.pathOrURL = tokenizer.next();
-    if (!tokenizer.hasNext())
-        return result;
-
     std::string arg = tokenizer.next();
-    if (arg != std::string("-p") && arg != std::string("--pixel-test"))
-        die(inputLine);
-    result.shouldDumpPixels = true;
-
-    if (tokenizer.hasNext())
-        result.expectedPixelHash = tokenizer.next();
-
+    result.pathOrURL = arg;
+    while (tokenizer.hasNext()) {
+        arg = tokenizer.next();
+        if (arg == std::string("--timeout")) {
+            std::string timeoutToken = tokenizer.next();
+            result.timeout = atoi(timeoutToken.c_str());
+        } else if (arg == std::string("-p") || arg == std::string("--pixel-test")) {
+            result.shouldDumpPixels = true;
+            if (tokenizer.hasNext())
+                result.expectedPixelHash = tokenizer.next();
+        } else
+            die(inputLine);
+    }
     return result;
 }
 
@@ -598,8 +700,10 @@ bool TestController::runTest(const char* inputLine)
     m_state = RunningTest;
 
     m_currentInvocation = adoptPtr(new TestInvocation(command.pathOrURL));
-    if (command.shouldDumpPixels)
+    if (command.shouldDumpPixels || m_shouldDumpPixelsForAllTests)
         m_currentInvocation->setIsPixelTest(command.expectedPixelHash);
+    if (command.timeout > 0)
+        m_currentInvocation->setCustomTimeout(command.timeout);
 
     m_currentInvocation->invoke();
     m_currentInvocation.clear();
@@ -626,7 +730,7 @@ void TestController::runTestingServerLoop()
 void TestController::run()
 {
     if (!resetStateToConsistentValues()) {
-        TestInvocation::dumpWebProcessUnresponsiveness("Failed to reset to consistent state before the first test");
+        m_currentInvocation->dumpWebProcessUnresponsiveness();
         return;
     }
 
@@ -651,6 +755,9 @@ void TestController::runUntil(bool& done, TimeoutDuration timeoutDuration)
         case LongTimeout:
             timeout = m_longTimeout;
             break;
+        case CustomTimeout:
+            timeout = m_timeout;
+            break;
         case NoTimeout:
         default:
             timeout = m_noTimeout;
@@ -673,10 +780,66 @@ void TestController::didReceiveSynchronousMessageFromInjectedBundle(WKContextRef
     *returnData = static_cast<TestController*>(const_cast<void*>(clientInfo))->didReceiveSynchronousMessageFromInjectedBundle(messageName, messageBody).leakRef();
 }
 
+void TestController::didReceiveKeyDownMessageFromInjectedBundle(WKDictionaryRef messageBodyDictionary, bool synchronous)
+{
+    WKRetainPtr<WKStringRef> keyKey = adoptWK(WKStringCreateWithUTF8CString("Key"));
+    WKStringRef key = static_cast<WKStringRef>(WKDictionaryGetItemForKey(messageBodyDictionary, keyKey.get()));
+
+    WKRetainPtr<WKStringRef> modifiersKey = adoptWK(WKStringCreateWithUTF8CString("Modifiers"));
+    WKEventModifiers modifiers = static_cast<WKEventModifiers>(WKUInt64GetValue(static_cast<WKUInt64Ref>(WKDictionaryGetItemForKey(messageBodyDictionary, modifiersKey.get()))));
+
+    WKRetainPtr<WKStringRef> locationKey = adoptWK(WKStringCreateWithUTF8CString("Location"));
+    unsigned location = static_cast<unsigned>(WKUInt64GetValue(static_cast<WKUInt64Ref>(WKDictionaryGetItemForKey(messageBodyDictionary, locationKey.get()))));
+
+    if (synchronous)
+        WKPageSetShouldSendEventsSynchronously(mainWebView()->page(), true);
+
+    m_eventSenderProxy->keyDown(key, modifiers, location);
+
+    if (synchronous)
+        WKPageSetShouldSendEventsSynchronously(mainWebView()->page(), false);
+}
+
 void TestController::didReceiveMessageFromInjectedBundle(WKStringRef messageName, WKTypeRef messageBody)
 {
+#if PLATFORM(MAC) || PLATFORM(QT) || PLATFORM(GTK) || PLATFORM(EFL)
+    if (WKStringIsEqualToUTF8CString(messageName, "EventSender")) {
+        ASSERT(WKGetTypeID(messageBody) == WKDictionaryGetTypeID());
+        WKDictionaryRef messageBodyDictionary = static_cast<WKDictionaryRef>(messageBody);
+
+        WKRetainPtr<WKStringRef> subMessageKey(AdoptWK, WKStringCreateWithUTF8CString("SubMessage"));
+        WKStringRef subMessageName = static_cast<WKStringRef>(WKDictionaryGetItemForKey(messageBodyDictionary, subMessageKey.get()));
+
+        if (WKStringIsEqualToUTF8CString(subMessageName, "MouseDown") || WKStringIsEqualToUTF8CString(subMessageName, "MouseUp")) {
+            WKRetainPtr<WKStringRef> buttonKey = adoptWK(WKStringCreateWithUTF8CString("Button"));
+            unsigned button = static_cast<unsigned>(WKUInt64GetValue(static_cast<WKUInt64Ref>(WKDictionaryGetItemForKey(messageBodyDictionary, buttonKey.get()))));
+
+            WKRetainPtr<WKStringRef> modifiersKey = adoptWK(WKStringCreateWithUTF8CString("Modifiers"));
+            WKEventModifiers modifiers = static_cast<WKEventModifiers>(WKUInt64GetValue(static_cast<WKUInt64Ref>(WKDictionaryGetItemForKey(messageBodyDictionary, modifiersKey.get()))));
+
+            // Forward to WebProcess
+            WKPageSetShouldSendEventsSynchronously(mainWebView()->page(), false);
+            if (WKStringIsEqualToUTF8CString(subMessageName, "MouseDown"))
+                m_eventSenderProxy->mouseDown(button, modifiers);
+            else
+                m_eventSenderProxy->mouseUp(button, modifiers);
+
+            return;
+        }
+
+        if (WKStringIsEqualToUTF8CString(subMessageName, "KeyDown")) {
+            didReceiveKeyDownMessageFromInjectedBundle(messageBodyDictionary, false);
+
+            return;
+        }
+
+        ASSERT_NOT_REACHED();
+    }
+#endif
+
     if (!m_currentInvocation)
         return;
+
     m_currentInvocation->didReceiveMessageFromInjectedBundle(messageName, messageBody);
 }
 
@@ -691,19 +854,8 @@ WKRetainPtr<WKTypeRef> TestController::didReceiveSynchronousMessageFromInjectedB
         WKStringRef subMessageName = static_cast<WKStringRef>(WKDictionaryGetItemForKey(messageBodyDictionary, subMessageKey.get()));
 
         if (WKStringIsEqualToUTF8CString(subMessageName, "KeyDown")) {
-            WKRetainPtr<WKStringRef> keyKey = adoptWK(WKStringCreateWithUTF8CString("Key"));
-            WKStringRef key = static_cast<WKStringRef>(WKDictionaryGetItemForKey(messageBodyDictionary, keyKey.get()));
+            didReceiveKeyDownMessageFromInjectedBundle(messageBodyDictionary, true);
 
-            WKRetainPtr<WKStringRef> modifiersKey = adoptWK(WKStringCreateWithUTF8CString("Modifiers"));
-            WKEventModifiers modifiers = static_cast<WKEventModifiers>(WKUInt64GetValue(static_cast<WKUInt64Ref>(WKDictionaryGetItemForKey(messageBodyDictionary, modifiersKey.get()))));
-
-            WKRetainPtr<WKStringRef> locationKey = adoptWK(WKStringCreateWithUTF8CString("Location"));
-            unsigned location = static_cast<unsigned>(WKUInt64GetValue(static_cast<WKUInt64Ref>(WKDictionaryGetItemForKey(messageBodyDictionary, locationKey.get()))));
-
-            // Forward to WebProcess
-            WKPageSetShouldSendEventsSynchronously(mainWebView()->page(), true);
-            m_eventSenderProxy->keyDown(key, modifiers, location);
-            WKPageSetShouldSendEventsSynchronously(mainWebView()->page(), false);
             return 0;
         }
 
@@ -890,6 +1042,11 @@ void TestController::didFinishLoadForFrame(WKPageRef page, WKFrameRef frame, WKT
     static_cast<TestController*>(const_cast<void*>(clientInfo))->didFinishLoadForFrame(page, frame);
 }
 
+void TestController::didReceiveAuthenticationChallengeInFrame(WKPageRef page, WKFrameRef frame, WKAuthenticationChallengeRef authenticationChallenge, const void *clientInfo)
+{
+    static_cast<TestController*>(const_cast<void*>(clientInfo))->didReceiveAuthenticationChallengeInFrame(page, frame, authenticationChallenge);
+}
+
 void TestController::processDidCrash(WKPageRef page, const void* clientInfo)
 {
     static_cast<TestController*>(const_cast<void*>(clientInfo))->processDidCrash();
@@ -919,6 +1076,26 @@ void TestController::didFinishLoadForFrame(WKPageRef page, WKFrameRef frame)
     shared().notifyDone();
 }
 
+void TestController::didReceiveAuthenticationChallengeInFrame(WKPageRef page, WKFrameRef frame, WKAuthenticationChallengeRef authenticationChallenge)
+{
+    String message;
+    if (!m_handlesAuthenticationChallenges)
+        message = "<unknown> - didReceiveAuthenticationChallenge - Simulating cancelled authentication sheet\n";
+    else
+        message = String::format("<unknown> - didReceiveAuthenticationChallenge - Responding with %s:%s\n", m_authenticationUsername.utf8().data(), m_authenticationPassword.utf8().data());
+    m_currentInvocation->outputText(message);
+
+    WKAuthenticationDecisionListenerRef decisionListener = WKAuthenticationChallengeGetDecisionListener(authenticationChallenge);
+    if (!m_handlesAuthenticationChallenges) {
+        WKAuthenticationDecisionListenerUseCredential(decisionListener, 0);
+        return;
+    }
+    WKRetainPtr<WKStringRef> username(AdoptWK, WKStringCreateWithUTF8CString(m_authenticationUsername.utf8().data()));
+    WKRetainPtr<WKStringRef> password(AdoptWK, WKStringCreateWithUTF8CString(m_authenticationPassword.utf8().data()));
+    WKRetainPtr<WKCredentialRef> credential(AdoptWK, WKCredentialCreate(username.get(), password.get(), kWKCredentialPersistenceForSession));
+    WKAuthenticationDecisionListenerUseCredential(decisionListener, credential.get());
+}
+
 void TestController::processDidCrash()
 {
     // This function can be called multiple times when crash logs are being saved on Windows, so
@@ -938,4 +1115,106 @@ void TestController::processDidCrash()
         exit(1);
 }
 
+void TestController::simulateWebNotificationClick(uint64_t notificationID)
+{
+    m_webNotificationProvider.simulateWebNotificationClick(notificationID);
+}
+
+void TestController::setGeolocationPermission(bool enabled)
+{
+    m_isGeolocationPermissionSet = true;
+    m_isGeolocationPermissionAllowed = enabled;
+    decidePolicyForGeolocationPermissionRequestIfPossible();
+}
+
+void TestController::setMockGeolocationPosition(double latitude, double longitude, double accuracy, bool providesAltitude, double altitude, bool providesAltitudeAccuracy, double altitudeAccuracy, bool providesHeading, double heading, bool providesSpeed, double speed)
+{
+    m_geolocationProvider->setPosition(latitude, longitude, accuracy, providesAltitude, altitude, providesAltitudeAccuracy, altitudeAccuracy, providesHeading, heading, providesSpeed, speed);
+}
+
+void TestController::setMockGeolocationPositionUnavailableError(WKStringRef errorMessage)
+{
+    m_geolocationProvider->setPositionUnavailableError(errorMessage);
+}
+
+void TestController::handleGeolocationPermissionRequest(WKGeolocationPermissionRequestRef geolocationPermissionRequest)
+{
+    m_geolocationPermissionRequests.append(geolocationPermissionRequest);
+    decidePolicyForGeolocationPermissionRequestIfPossible();
+}
+
+void TestController::setCustomPolicyDelegate(bool enabled, bool permissive)
+{
+    m_policyDelegateEnabled = enabled;
+    m_policyDelegatePermissive = permissive;
+}
+
+void TestController::setVisibilityState(WKPageVisibilityState visibilityState, bool isInitialState)
+{
+    WKPageSetVisibilityState(m_mainWebView->page(), visibilityState, isInitialState);
+}
+
+void TestController::decidePolicyForGeolocationPermissionRequestIfPossible()
+{
+    if (!m_isGeolocationPermissionSet)
+        return;
+
+    for (size_t i = 0; i < m_geolocationPermissionRequests.size(); ++i) {
+        WKGeolocationPermissionRequestRef permissionRequest = m_geolocationPermissionRequests[i].get();
+        if (m_isGeolocationPermissionAllowed)
+            WKGeolocationPermissionRequestAllow(permissionRequest);
+        else
+            WKGeolocationPermissionRequestDeny(permissionRequest);
+    }
+    m_geolocationPermissionRequests.clear();
+}
+
+void TestController::decidePolicyForNotificationPermissionRequest(WKPageRef page, WKSecurityOriginRef origin, WKNotificationPermissionRequestRef request, const void*)
+{
+    TestController::shared().decidePolicyForNotificationPermissionRequest(page, origin, request);
+}
+
+void TestController::decidePolicyForNotificationPermissionRequest(WKPageRef, WKSecurityOriginRef, WKNotificationPermissionRequestRef request)
+{
+    WKNotificationPermissionRequestAllow(request);
+}
+
+void TestController::unavailablePluginButtonClicked(WKPageRef, WKPluginUnavailabilityReason, WKStringRef, WKStringRef, WKStringRef, const void*)
+{
+    printf("MISSING PLUGIN BUTTON PRESSED\n");
+}
+
+void TestController::decidePolicyForNavigationAction(WKPageRef, WKFrameRef, WKFrameNavigationType, WKEventModifiers, WKEventMouseButton, WKURLRequestRef, WKFramePolicyListenerRef listener, WKTypeRef, const void* clientInfo)
+{
+    static_cast<TestController*>(const_cast<void*>(clientInfo))->decidePolicyForNavigationAction(listener);
+}
+
+void TestController::decidePolicyForNavigationAction(WKFramePolicyListenerRef listener)
+{
+    if (m_policyDelegateEnabled && !m_policyDelegatePermissive) {
+        WKFramePolicyListenerIgnore(listener);
+        return;
+    }
+
+    WKFramePolicyListenerUse(listener);
+}
+
+void TestController::decidePolicyForResponse(WKPageRef, WKFrameRef frame, WKURLResponseRef response, WKURLRequestRef, WKFramePolicyListenerRef listener, WKTypeRef, const void* clientInfo)
+{
+    static_cast<TestController*>(const_cast<void*>(clientInfo))->decidePolicyForResponse(frame, response, listener);
+}
+
+void TestController::decidePolicyForResponse(WKFrameRef frame, WKURLResponseRef response, WKFramePolicyListenerRef listener)
+{
+    // Even though Response was already checked by WKBundlePagePolicyClient, the check did not include plugins
+    // so we have to re-check again.
+    WKRetainPtr<WKStringRef> wkMIMEType(AdoptWK, WKURLResponseCopyMIMEType(response));
+    if (WKFrameCanShowMIMEType(frame, wkMIMEType.get())) {
+        WKFramePolicyListenerUse(listener);
+        return;
+    }
+
+    WKFramePolicyListenerIgnore(listener);
+}
+
 } // namespace WTR
